#include <Python.h>

#include <Numeric/arrayobject.h>

#ifndef __PETSCKSP_H
#include <petscksp.h>
#endif

enum ReferenceType {STOLEN_REF, WEAK_REF, NEW_REF};

struct PETSc_KSP_PythonObject {
  PyObject_HEAD;
  KSP ior;
  enum ReferenceType refType;
  PyObject *weakRefList;
};

/* Need to forward declare the type structure */
static PyTypeObject SIDLPythonObjectType;

static PyObject *sidl_new(PyTypeObject *type, PetscObject ior, enum ReferenceType refType)
{
  PyObject *return_value = NULL;
  
  if (ior)
  {
    struct PETSc_KSP_PythonObject *self = (struct PETSc_KSP_PythonObject *) type->tp_alloc(type, 0);
    
    self->weakRefList = NULL;
    if (self)
    {
      self->ior = (KSP) ior;
      self->refType = refType;
      if (refType == NEW_REF)
      {
        PetscObjectReference(ior);
      }
      return_value = (PyObject *) self;
    }
    else
    {
      PyErr_SetString(PyExc_MemoryError, (char *) "Unable to allocate interface object");
    }
  }
  else
  {
    return_value = Py_None;
    Py_INCREF(return_value);
  }
  return return_value;
}

static PyObject *opaque__wrap_ior(void *ptr)
{
  if (ptr)
  {
    return PyCObject_FromVoidPtr(ptr, NULL);
  }
  else
  {
    Py_INCREF(Py_None);
    return Py_None;
  }
}

static PyObject *PETSc_Mat__wrap_ior(Mat ior)
{
  PyObject *module = PyImport_ImportModule((char *) "PETSc.Mat");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Mat");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module PETSc.Mat");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type PETSc.Mat");
    return 0;
  }
  return sidl_new(type, (PetscObject) ior, STOLEN_REF);
}

static PyObject *PETSc_PC__wrap_ior(PC ior)
{
  PyObject *module = PyImport_ImportModule((char *) "PETSc.PC");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "PC");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module PETSc.PC");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type PETSc.PC");
    return 0;
  }
  return sidl_new(type, (PetscObject) ior, STOLEN_REF);
}

static PyObject *PETSc_KSP__wrap_ior(KSP ior)
{
  PyObject *module = PyImport_ImportModule((char *) "PETSc.KSP");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "KSP");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module PETSc.KSP");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type PETSc.KSP");
    return 0;
  }
  return sidl_new(type, (PetscObject) ior, STOLEN_REF);
}

static PyObject *PETSc_Vec__wrap_ior(Vec ior)
{
  PyObject *module = PyImport_ImportModule((char *) "PETSc.Vec");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Vec");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module PETSc.Vec");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type PETSc.Vec");
    return 0;
  }
  return sidl_new(type, (PetscObject) ior, STOLEN_REF);
}

static long PETSc_Mat__extract_ior(struct PETSc_KSP_PythonObject *obj, Mat *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (Mat) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "Mat"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.Mat");
    return 0;
  }
  return 1;
}

static long PETSc_PC__extract_ior(struct PETSc_KSP_PythonObject *obj, PC *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (PC) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "PC"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.PC");
    return 0;
  }
  return 1;
}

static long PETSc_KSP__extract_ior(struct PETSc_KSP_PythonObject *obj, KSP *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (KSP) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "KSP"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.KSP");
    return 0;
  }
  return 1;
}

static long PETSc_PetscObject__extract_ior(struct PETSc_KSP_PythonObject *obj, PetscObject *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (PetscObject) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "PetscObject"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.PetscObject");
    return 0;
  }
  return 1;
}

static long PETSc_PetscViewer__extract_ior(struct PETSc_KSP_PythonObject *obj, PetscViewer *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (PetscViewer) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "PetscViewer"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.PetscViewer");
    return 0;
  }
  return 1;
}

static long PETSc_Vec__extract_ior(struct PETSc_KSP_PythonObject *obj, Vec *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (Vec) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "Vec"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.Vec");
    return 0;
  }
  return 1;
}

static long opaque__extract_ior(PyObject *obj, void **ptr)
{
  if (PyCObject_Check(obj))
  {
    *ptr = PyCObject_AsVoidPtr(obj);
    return 1;
  }
  /* Why in the hell is there no Python interface to CObject? */
  if (PyObject_HasAttrString(obj, "handle"))
  {
    *ptr = PyLong_AsVoidPtr(PyObject_GetAttrString(obj, "handle"));
    return 1;
  }
  if (obj == Py_None)
  {
    *ptr = 0;
    return 1;
  }
  return 0;
}

#undef __FUNCT__

#define __FUNCT__ "PythonKSPConvergenceTest"

static int32_t PythonKSPConvergenceTest(KSP ksp, int32_t iterationNumber, double residualNorm, KSPConvergedReason *convergedReason, PyObject *ctx)
{
  PyObject *_method = (PyObject *) ctx;
  long _local_iterationNumber = 0;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  long _local_convergedReason = 0;
  
  if (_method == Py_None)
  {
    return 0;
  }
  Py_INCREF(_method);
  if (ksp)
  {
    PetscObjectReference(((PetscObject) ksp));
  }
  /* Convert int32_t to long */
  _local_iterationNumber = iterationNumber;
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&ld)", (void *) PETSc_KSP__wrap_ior, ksp, _local_iterationNumber, residualNorm);
  if (!_args)
  {
    SETERRQ(PETSC_ERR_LIB, "Unable to parse arguments for PETSc.KSP.PythonKSPConvergenceTest");
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (!_pyReturn)
  {
    PyErr_Print();
    return _return_value;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTuple(_pyReturn, (char *) "ll", &_local__return_value, &_local_convergedReason);
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_okay)
  {
    SETERRQ(PETSC_ERR_LIB, "Could not parse return value for PETSc.KSP.PythonKSPConvergenceTest");
  }
  /* Convert long to int32_t */
  _return_value = _local__return_value;
  /* Convert long to KSPConvergedReason */
  *convergedReason = _local_convergedReason;
  /* Cleanup local return value */
  if (_pyReturn && (_pyReturn != Py_None))
  {
    Py_DECREF(_pyReturn);
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return _return_value;
}

static PyObject *PETSc_PetscObject_getType(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getType");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getType");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPGetType(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setType(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "typeName",
    NULL};
  long _okay = 0;
  const char *typeName = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setType");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setType");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &typeName);
  /* Copying the Python string to IOR controlled memory */
  typeName = typeName ? strcpy(malloc(strlen(typeName) + 1), typeName) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPSetType(_self_ior, typeName);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) typeName);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_getOptionsPrefix(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getOptionsPrefix");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getOptionsPrefix");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPGetOptionsPrefix(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setOptionsPrefix(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "prefix",
    NULL};
  long _okay = 0;
  const char *prefix = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setOptionsPrefix");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setOptionsPrefix");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &prefix);
  /* Copying the Python string to IOR controlled memory */
  prefix = prefix ? strcpy(malloc(strlen(prefix) + 1), prefix) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPSetOptionsPrefix(_self_ior, prefix);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) prefix);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_appendOptionsPrefix(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "prefix",
    NULL};
  long _okay = 0;
  const char *prefix = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for appendOptionsPrefix");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in appendOptionsPrefix");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &prefix);
  /* Copying the Python string to IOR controlled memory */
  prefix = prefix ? strcpy(malloc(strlen(prefix) + 1), prefix) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPAppendOptionsPrefix(_self_ior, prefix);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) prefix);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setFromOptions(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setFromOptions");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setFromOptions");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPSetFromOptions(_self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setUp(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setUp");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setUp");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPSetUp(_self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_view(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "viewer",
    NULL};
  long _okay = 0;
  PetscViewer viewer = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for view");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in view");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_PetscViewer__extract_ior, &viewer);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPView(_self_ior, viewer);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_getObject(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  void *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getObject");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getObject");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) opaque__wrap_ior, _return_value);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setObject(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "object",
    NULL};
  long _okay = 0;
  void *object = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setObject");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setObject");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) opaque__extract_ior, &object);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  if ((((struct PETSc_KSP_PythonObject *) _self)->refType != WEAK_REF) && (((struct PETSc_KSP_PythonObject *) _self)->ior))
  {
    KSPDestroy(((struct PETSc_KSP_PythonObject *) _self)->ior);
  }
  ((struct PETSc_KSP_PythonObject *) _self)->ior = object;
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getReferenceCount(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getReferenceCount");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getReferenceCount");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectGetReference((PetscObject) _self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_addReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for addReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in addReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectReference((PetscObject) _self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_deleteReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for deleteReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in deleteReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectDereference((PetscObject) _self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getClassName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  const char *_return_value = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getClassName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getClassName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = ((PetscObject) _self_ior)->class_name;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectGetName((PetscObject) _self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_setName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "name",
    NULL};
  long _okay = 0;
  const char *name = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &name);
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectSetName((PetscObject) _self_ior, name);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) name);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_isInstanceOf(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "typeName",
    NULL};
  long _okay = 0;
  const char *typeName = NULL;
  PetscTruth _return_value = 0;
  PetscTruth _local__return_value = 0;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isInstanceOf");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in isInstanceOf");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &typeName);
  /* Copying the Python string to IOR controlled memory */
  typeName = typeName ? strcpy(malloc(strlen(typeName) + 1), typeName) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = !strcmp(((PetscObject) _self_ior)->class_name, typeName);
  /* Convert PetscTruth to PetscTruth */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local__return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) typeName);
  return _pyReturn;
}

static PyObject *PETSc_KSP_solve(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "b",
    (char *) "x",
    NULL};
  long _okay = 0;
  Vec b = 0;
  Vec x = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for solve");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in solve");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&", _kwlist, (void *) PETSc_Vec__extract_ior, &b, (void *) PETSc_Vec__extract_ior, &x);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPSolve(_self_ior, b, x);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_KSP_solveTranspose(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "b",
    (char *) "x",
    NULL};
  long _okay = 0;
  Vec b = 0;
  Vec x = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for solveTranspose");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in solveTranspose");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&", _kwlist, (void *) PETSc_Vec__extract_ior, &b, (void *) PETSc_Vec__extract_ior, &x);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPSolveTranspose(_self_ior, b, x);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_KSP_getOperators(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  Mat A = 0;
  Mat M = 0;
  MatStructure flag = 0;
  long _local_flag = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getOperators");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getOperators");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPGetOperators(_self_ior, &A, &M, &flag);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert MatStructure to long */
  _local_flag = flag;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "(O&O&l)", (void *) PETSc_Mat__wrap_ior, A, (void *) PETSc_Mat__wrap_ior, M, _local_flag);
  if (A)
  {
    PetscObjectReference(((PetscObject) A));
  }
  if (M)
  {
    PetscObjectReference(((PetscObject) M));
  }
  return _pyReturn;
}

static PyObject *PETSc_KSP_setOperators(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "A",
    (char *) "M",
    (char *) "flag",
    NULL};
  long _okay = 0;
  Mat A = 0;
  Mat M = 0;
  MatStructure flag = 0;
  long _local_flag = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setOperators");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setOperators");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&l", _kwlist, (void *) PETSc_Mat__extract_ior, &A, (void *) PETSc_Mat__extract_ior, &M, &_local_flag);
  /* Convert long to MatStructure */
  flag = _local_flag;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPSetOperators(_self_ior, A, M, flag);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_KSP_getPC(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  PC _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getPC");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getPC");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPGetPC(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) PETSc_PC__wrap_ior, _return_value);
  if (_return_value)
  {
    PetscObjectReference(((PetscObject) _return_value));
  }
  return _pyReturn;
}

static PyObject *PETSc_KSP_setPC(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "preconditioner",
    NULL};
  long _okay = 0;
  PC preconditioner = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setPC");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setPC");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_PC__extract_ior, &preconditioner);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPSetPC(_self_ior, preconditioner);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_KSP_getPreconditionerSide(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  PCSide _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getPreconditionerSide");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getPreconditionerSide");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPGetPreconditionerSide(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert PCSide to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *PETSc_KSP_setPreconditionerSide(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "side",
    NULL};
  long _okay = 0;
  PCSide side = 0;
  long _local_side = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setPreconditionerSide");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setPreconditionerSide");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "l", _kwlist, &_local_side);
  /* Convert long to PCSide */
  side = _local_side;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPSetPreconditionerSide(_self_ior, side);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_KSP_getTolerances(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  double relativeTol = 0.0;
  double absoluteTol = 0.0;
  double divergenceTol = 0.0;
  int32_t maxIterations = 0;
  long _local_maxIterations = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getTolerances");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getTolerances");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPGetTolerances(_self_ior, &relativeTol, &absoluteTol, &divergenceTol, &maxIterations);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local_maxIterations = maxIterations;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "(dddl)", relativeTol, absoluteTol, divergenceTol, _local_maxIterations);
  return _pyReturn;
}

static PyObject *PETSc_KSP_setTolerances(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "relativeTol",
    (char *) "absoluteTol",
    (char *) "divergenceTol",
    (char *) "maxIterations",
    NULL};
  long _okay = 0;
  double relativeTol = 0.0;
  double absoluteTol = 0.0;
  double divergenceTol = 0.0;
  int32_t maxIterations = 0;
  long _local_maxIterations = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setTolerances");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setTolerances");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "dddl", _kwlist, &relativeTol, &absoluteTol, &divergenceTol, &_local_maxIterations);
  /* Convert long to int32_t */
  maxIterations = _local_maxIterations;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPSetTolerances(_self_ior, relativeTol, absoluteTol, divergenceTol, maxIterations);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_KSP_getInitialGuessNonzero(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  PetscTruth _return_value = 0;
  PetscTruth _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getInitialGuessNonzero");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getInitialGuessNonzero");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPGetInitialGuessNonzero(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert PetscTruth to PetscTruth */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local__return_value);
  return _pyReturn;
}

static PyObject *PETSc_KSP_setInitialGuessNonzero(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "nonzeroGuess",
    NULL};
  long _okay = 0;
  PetscTruth nonzeroGuess = 0;
  PetscTruth _local_nonzeroGuess = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setInitialGuessNonzero");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setInitialGuessNonzero");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "i", _kwlist, &_local_nonzeroGuess);
  /* Convert PetscTruth to PetscTruth */
  nonzeroGuess = _local_nonzeroGuess;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPSetInitialGuessNonzero(_self_ior, nonzeroGuess);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_KSP_getComputeEigenvalues(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  PetscTruth _return_value = 0;
  PetscTruth _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getComputeEigenvalues");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getComputeEigenvalues");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPGetComputeEigenvalues(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert PetscTruth to PetscTruth */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local__return_value);
  return _pyReturn;
}

static PyObject *PETSc_KSP_setComputeEigenvalues(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "computeEigenvalues",
    NULL};
  long _okay = 0;
  PetscTruth computeEigenvalues = 0;
  PetscTruth _local_computeEigenvalues = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setComputeEigenvalues");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setComputeEigenvalues");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "i", _kwlist, &_local_computeEigenvalues);
  /* Convert PetscTruth to PetscTruth */
  computeEigenvalues = _local_computeEigenvalues;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPSetComputeEigenvalues(_self_ior, computeEigenvalues);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_KSP_getComputeSingularValues(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  PetscTruth _return_value = 0;
  PetscTruth _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getComputeSingularValues");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getComputeSingularValues");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPGetComputeSingularValues(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert PetscTruth to PetscTruth */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local__return_value);
  return _pyReturn;
}

static PyObject *PETSc_KSP_setComputeSingularValues(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "computeSingularValues",
    NULL};
  long _okay = 0;
  PetscTruth computeSingularValues = 0;
  PetscTruth _local_computeSingularValues = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setComputeSingularValues");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setComputeSingularValues");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "i", _kwlist, &_local_computeSingularValues);
  /* Convert PetscTruth to PetscTruth */
  computeSingularValues = _local_computeSingularValues;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPSetComputeSingularValues(_self_ior, computeSingularValues);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_KSP_setNormType(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "normType",
    NULL};
  long _okay = 0;
  KSPNormType normType = 0;
  long _local_normType = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setNormType");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setNormType");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "l", _kwlist, &_local_normType);
  /* Convert long to KSPNormType */
  normType = _local_normType;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPSetNormType(_self_ior, normType);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_KSP_setConvergenceTest(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "convergenceTest",
    NULL};
  long _okay = 0;
  PyObject *convergenceTest = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setConvergenceTest");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setConvergenceTest");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O", _kwlist, &convergenceTest);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPSetConvergenceTest(_self_ior, PythonKSPConvergenceTest, convergenceTest);
  /* PETSc must hold a reference to the method object (these leak right now) */
  Py_INCREF(convergenceTest);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_KSP_getRhs(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  Vec _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getRhs");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getRhs");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPGetRhs(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) PETSc_Vec__wrap_ior, _return_value);
  if (_return_value)
  {
    PetscObjectReference(((PetscObject) _return_value));
  }
  return _pyReturn;
}

static PyObject *PETSc_KSP_getSolution(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  Vec _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getSolution");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getSolution");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPGetSolution(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) PETSc_Vec__wrap_ior, _return_value);
  if (_return_value)
  {
    PetscObjectReference(((PetscObject) _return_value));
  }
  return _pyReturn;
}

static PyObject *PETSc_KSP_getResidualNorm(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  double _return_value = 0.0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getResidualNorm");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getResidualNorm");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPGetResidualNorm(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "d", _return_value);
  return _pyReturn;
}

static PyObject *PETSc_KSP_getIterationNumber(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getIterationNumber");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getIterationNumber");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPGetIterationNumber(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *PETSc_KSP_getConvergedReason(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  KSPConvergedReason _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getConvergedReason");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getConvergedReason");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = KSPGetConvergedReason(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert KSPConvergedReason to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *PETSc_KSP_KSP(PyObject *_type, PyObject *_args, PyObject *_kwdict)
{
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  long _okay = 0;
  KSP obj = 0;
  PetscTruth created = 0;
  KSP _return_value = 0;
  PyObject *_pyReturn = NULL;
  
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "|O&", _kwlist, (void *) PETSc_KSP__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  if (obj)
  {
    created = 0;
  }
  else
  {
    created = 1;
    KSPCreate(PETSC_COMM_WORLD, &obj);
  }
  _return_value = obj;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) PETSc_KSP__wrap_ior, _return_value);
  if (_return_value)
  {
    PetscObjectReference(((PetscObject) _return_value));
  }
  if (created)
  {
    PetscObjectDereference((PetscObject) obj);
  }
  return _pyReturn;
}

static PyObject *isSame(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  /* Retrieve the IOR pointer */
  KSP _self_ior = ((struct PETSc_KSP_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  long _okay = 0;
  PetscObject obj = 0;
  PetscTruth _return_value = 0;
  PetscTruth _local__return_value = 0;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isSame");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in isSame");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_PetscObject__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = (_self_ior == (KSP) obj);
  /* Convert PetscTruth to PetscTruth */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local__return_value);
  return _pyReturn;
}

static struct PyMethodDef _KSPObjectMethods[] = {
  {(char *) "getType", (PyCFunction) PETSc_PetscObject_getType, (METH_VARARGS | METH_KEYWORDS), (char *) "getType()\n\
RETURNS (return string _return_value)\n\
 Get the implementation type"},
  {(char *) "setType", (PyCFunction) PETSc_PetscObject_setType, (METH_VARARGS | METH_KEYWORDS), (char *) "setType(in string typeName)\n\
RETURNS None\n\
\n\
 Set the implementation type"},
  {(char *) "getOptionsPrefix", (PyCFunction) PETSc_PetscObject_getOptionsPrefix, (METH_VARARGS | METH_KEYWORDS), (char *) "getOptionsPrefix()\n\
RETURNS (return string _return_value)\n\
 Get the prefix appended to options directed to this object"},
  {(char *) "setOptionsPrefix", (PyCFunction) PETSc_PetscObject_setOptionsPrefix, (METH_VARARGS | METH_KEYWORDS), (char *) "setOptionsPrefix(in string prefix)\n\
RETURNS None\n\
\n\
 Set the prefix appended to options directed to this object"},
  {(char *) "appendOptionsPrefix", (PyCFunction) PETSc_PetscObject_appendOptionsPrefix, (METH_VARARGS | METH_KEYWORDS), (char *) "appendOptionsPrefix(in string prefix)\n\
RETURNS None\n\
\n\
 Enlarge the prefix appended to options directed to this object"},
  {(char *) "setFromOptions", (PyCFunction) PETSc_PetscObject_setFromOptions, (METH_VARARGS | METH_KEYWORDS), (char *) "setFromOptions()\n\
RETURNS None\n\
\n\
 Configure the object using the PETSc options database"},
  {(char *) "setUp", (PyCFunction) PETSc_PetscObject_setUp, (METH_VARARGS | METH_KEYWORDS), (char *) "setUp()\n\
RETURNS None\n\
\n\
 Initiate the object setup phase"},
  {(char *) "view", (PyCFunction) PETSc_PetscObject_view, (METH_VARARGS | METH_KEYWORDS), (char *) "view(in PETSc.PetscViewer viewer)\n\
RETURNS None\n\
\n\
 Output a representation of the object using "},
  {(char *) "getObject", (PyCFunction) PETSc_PetscObject_getObject, (METH_VARARGS | METH_KEYWORDS), (char *) "getObject()\n\
RETURNS (return opaque _return_value)\n\
 Get the PETSc object pointer"},
  {(char *) "setObject", (PyCFunction) PETSc_PetscObject_setObject, (METH_VARARGS | METH_KEYWORDS), (char *) "setObject(in opaque object)\n\
RETURNS None\n\
\n\
 Set the PETSc object pointer"},
  {(char *) "getReferenceCount", (PyCFunction) ASE_BaseInterface_getReferenceCount, (METH_VARARGS | METH_KEYWORDS), (char *) "getReferenceCount()\n\
RETURNS (return int _return_value)\n\
 Retrieve the intrinsic reference count of the underlying object"},
  {(char *) "addReference", (PyCFunction) ASE_BaseInterface_addReference, (METH_VARARGS | METH_KEYWORDS), (char *) "addReference()\n\
RETURNS None\n\
\n\
 Add one to the intrinsic reference count of the underlying object - Objects continue to exist as long as the reference count is positive - Clients should call this method whenever they create another ongoing reference to an object or interface"},
  {(char *) "deleteReference", (PyCFunction) ASE_BaseInterface_deleteReference, (METH_VARARGS | METH_KEYWORDS), (char *) "deleteReference()\n\
RETURNS None\n\
\n\
 Decrease by one the intrinsic reference count of the underlying object - Delete the object if the reference count is non-positive - Clients should call this method whenever they remove a reference to an object or interface"},
  {(char *) "getClassName", (PyCFunction) ASE_BaseInterface_getClassName, (METH_VARARGS | METH_KEYWORDS), (char *) "getClassName()\n\
RETURNS (return string _return_value)\n\
 Return the type name of the underlying class object"},
  {(char *) "getName", (PyCFunction) ASE_BaseInterface_getName, (METH_VARARGS | METH_KEYWORDS), (char *) "getName()\n\
RETURNS (return string _return_value)\n\
 Retrieve the object name"},
  {(char *) "setName", (PyCFunction) ASE_BaseInterface_setName, (METH_VARARGS | METH_KEYWORDS), (char *) "setName(in string name)\n\
RETURNS None\n\
\n\
 Set the object name"},
  {(char *) "isInstanceOf", (PyCFunction) ASE_BaseInterface_isInstanceOf, (METH_VARARGS | METH_KEYWORDS), (char *) "isInstanceOf(in string typeName)\n\
RETURNS (return bool _return_value)\n\
 Return <code>true</code> if this object is an instance of the specified type - This routine will return <code>true</code> if and only if a cast to the specified type would succeed"},
  {(char *) "solve", (PyCFunction) PETSc_KSP_solve, (METH_VARARGS | METH_KEYWORDS), (char *) "solve(in PETSc.Vec b, in PETSc.Vec x)\n\
RETURNS None\n\
\n\
 Solve the linear system A x = b"},
  {(char *) "solveTranspose", (PyCFunction) PETSc_KSP_solveTranspose, (METH_VARARGS | METH_KEYWORDS), (char *) "solveTranspose(in PETSc.Vec b, in PETSc.Vec x)\n\
RETURNS None\n\
\n\
 Solve the linear system A^T x = b"},
  {(char *) "getOperators", (PyCFunction) PETSc_KSP_getOperators, (METH_VARARGS | METH_KEYWORDS), (char *) "getOperators(out PETSc.Mat A, out PETSc.Mat M, out PETSc.MatStructure flag)\n\
RETURNS (out PETSc.Mat A, out PETSc.Mat M, out PETSc.MatStructure flag)\n\
 Get the matrix A and preconditioning matrix M"},
  {(char *) "setOperators", (PyCFunction) PETSc_KSP_setOperators, (METH_VARARGS | METH_KEYWORDS), (char *) "setOperators(in PETSc.Mat A, in PETSc.Mat M, in PETSc.MatStructure flag)\n\
RETURNS None\n\
\n\
 Set the matrix A and preconditioning matrix M"},
  {(char *) "getPC", (PyCFunction) PETSc_KSP_getPC, (METH_VARARGS | METH_KEYWORDS), (char *) "getPC()\n\
RETURNS (return PETSc.PC _return_value)\n\
 Get the preconditioner"},
  {(char *) "setPC", (PyCFunction) PETSc_KSP_setPC, (METH_VARARGS | METH_KEYWORDS), (char *) "setPC(in PETSc.PC preconditioner)\n\
RETURNS None\n\
\n\
 Set the preconditioner"},
  {(char *) "getPreconditionerSide", (PyCFunction) PETSc_KSP_getPreconditionerSide, (METH_VARARGS | METH_KEYWORDS), (char *) "getPreconditionerSide()\n\
RETURNS (return PETSc.PCSide _return_value)\n\
 Get the preconditioner application type"},
  {(char *) "setPreconditionerSide", (PyCFunction) PETSc_KSP_setPreconditionerSide, (METH_VARARGS | METH_KEYWORDS), (char *) "setPreconditionerSide(in PETSc.PCSide side)\n\
RETURNS None\n\
\n\
 Set the preconditioner application type"},
  {(char *) "getTolerances", (PyCFunction) PETSc_KSP_getTolerances, (METH_VARARGS | METH_KEYWORDS), (char *) "getTolerances(out double relativeTol, out double absoluteTol, out double divergenceTol, out int maxIterations)\n\
RETURNS (out double relativeTol, out double absoluteTol, out double divergenceTol, out int maxIterations)\n\
 Get the solution tolerances"},
  {(char *) "setTolerances", (PyCFunction) PETSc_KSP_setTolerances, (METH_VARARGS | METH_KEYWORDS), (char *) "setTolerances(in double relativeTol, in double absoluteTol, in double divergenceTol, in int maxIterations)\n\
RETURNS None\n\
\n\
 Set the solution tolerances"},
  {(char *) "getInitialGuessNonzero", (PyCFunction) PETSc_KSP_getInitialGuessNonzero, (METH_VARARGS | METH_KEYWORDS), (char *) "getInitialGuessNonzero()\n\
RETURNS (return bool _return_value)\n\
 Get the flag for a nonzero initial guess"},
  {(char *) "setInitialGuessNonzero", (PyCFunction) PETSc_KSP_setInitialGuessNonzero, (METH_VARARGS | METH_KEYWORDS), (char *) "setInitialGuessNonzero(in bool nonzeroGuess)\n\
RETURNS None\n\
\n\
 Set the flag for a nonzero initial guess"},
  {(char *) "getComputeEigenvalues", (PyCFunction) PETSc_KSP_getComputeEigenvalues, (METH_VARARGS | METH_KEYWORDS), (char *) "getComputeEigenvalues()\n\
RETURNS (return bool _return_value)\n\
 Get the flag for computing eigenvalues of the reduced matrix"},
  {(char *) "setComputeEigenvalues", (PyCFunction) PETSc_KSP_setComputeEigenvalues, (METH_VARARGS | METH_KEYWORDS), (char *) "setComputeEigenvalues(in bool computeEigenvalues)\n\
RETURNS None\n\
\n\
 Set the flag for computing eigenvalues of the reduced matrix"},
  {(char *) "getComputeSingularValues", (PyCFunction) PETSc_KSP_getComputeSingularValues, (METH_VARARGS | METH_KEYWORDS), (char *) "getComputeSingularValues()\n\
RETURNS (return bool _return_value)\n\
 Get the flag for computing singular values of the reduced matrix"},
  {(char *) "setComputeSingularValues", (PyCFunction) PETSc_KSP_setComputeSingularValues, (METH_VARARGS | METH_KEYWORDS), (char *) "setComputeSingularValues(in bool computeSingularValues)\n\
RETURNS None\n\
\n\
 Set the flag for computing singular values of the reduced matrix"},
  {(char *) "setNormType", (PyCFunction) PETSc_KSP_setNormType, (METH_VARARGS | METH_KEYWORDS), (char *) "setNormType(in PETSc.KSPNormType normType)\n\
RETURNS None\n\
\n\
 Set the type of norm used to check convergence of the solver"},
  {(char *) "setConvergenceTest", (PyCFunction) PETSc_KSP_setConvergenceTest, (METH_VARARGS | METH_KEYWORDS), (char *) "setConvergenceTest(in function convergenceTest)\n\
RETURNS None\n\
\n\
 Set the function which is used to check convergence of the solver   Must be called after the KSP type has been set so put this after   a call to KSPSetType(), or KSPSetFromOptions().   Calling sequence of convergence test:     reason convergenceTest(ksp, iterationNumber, residualNorm)"},
  {(char *) "getRhs", (PyCFunction) PETSc_KSP_getRhs, (METH_VARARGS | METH_KEYWORDS), (char *) "getRhs()\n\
RETURNS (return PETSc.Vec _return_value)\n\
 Get the rhs"},
  {(char *) "getSolution", (PyCFunction) PETSc_KSP_getSolution, (METH_VARARGS | METH_KEYWORDS), (char *) "getSolution()\n\
RETURNS (return PETSc.Vec _return_value)\n\
 Get the solution"},
  {(char *) "getResidualNorm", (PyCFunction) PETSc_KSP_getResidualNorm, (METH_VARARGS | METH_KEYWORDS), (char *) "getResidualNorm()\n\
RETURNS (return double _return_value)\n\
 Get the norm of the residual b - A x"},
  {(char *) "getIterationNumber", (PyCFunction) PETSc_KSP_getIterationNumber, (METH_VARARGS | METH_KEYWORDS), (char *) "getIterationNumber()\n\
RETURNS (return int _return_value)\n\
 Get the number of iterates performed"},
  {(char *) "getConvergedReason", (PyCFunction) PETSc_KSP_getConvergedReason, (METH_VARARGS | METH_KEYWORDS), (char *) "getConvergedReason()\n\
RETURNS (return PETSc.KSPConvergedReason _return_value)\n\
 Get the reason for solver termination"},
  {(char *) "isSame", (PyCFunction) isSame, (METH_VARARGS | METH_KEYWORDS), (char *) "isSame(in PETSc.PetscObject obj)\n\
RETURNS (return bool _return_value)\n\
"},
  {NULL, NULL}};

static struct PyMethodDef _KSPModuleMethods[] = {{NULL, NULL}};

static void sidl_dealloc(struct PETSc_KSP_PythonObject *self)
{
  if (self->weakRefList)
  {
    PyObject_ClearWeakRefs((PyObject *) self);
  }
  if ((self->refType != WEAK_REF) && (self->ior))
  {
    KSPDestroy(self->ior);
  }
  self->ob_type->tp_free(self);
}

static int sidl_cmp(struct PETSc_KSP_PythonObject *self, PyObject *arg)
{
  KSP ior1 = self->ior;
  PetscObject ior2 = (PetscObject) ((struct PETSc_KSP_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (arg->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for sidl_cmp");
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  _return_value = PyObject_Compare(o1, o2);
  Py_DECREF(o1);
  Py_DECREF(o2);
  return _return_value;
}

static long sidl_hash(struct PETSc_KSP_PythonObject *self)
{
  KSP ior = self->ior;
  PyObject *_hash_arg = NULL;
  long _return_value = 0;
  
  if (!ior)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_hash()");
    return _return_value;
  }
  _hash_arg = Py_BuildValue((char *) "i", ior);
  if (!_hash_arg)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not hash IOR pointer in sidl_hash()");
    return _return_value;
  }
  _return_value = PyObject_Hash(_hash_arg);
  Py_DECREF(_hash_arg);
  return _return_value;
}

static PyObject *sidl_str(struct PETSc_KSP_PythonObject *self)
{
  const char *_str = NULL;
  PyObject *_return_value = NULL;
  KSP ior = self->ior;
  
  KSPGetType(ior, (KSPType *) &_str);
  _return_value = PyString_FromString(_str);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyObject *sidl_richcmp(struct PETSc_KSP_PythonObject *self, PyObject *arg, int op)
{
  KSP ior1 = self->ior;
  PetscObject ior2 = (PetscObject) ((struct PETSc_KSP_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  PyObject *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (!PyObject_HasAttrString(arg, (char *) "getClassName"))
  {
    switch(op) {
      case Py_EQ:
        _return_value = Py_False;
        Py_INCREF(_return_value);
        break;
      case Py_NE:
        _return_value = Py_True;
        Py_INCREF(_return_value);
        break;
      case Py_LT:
      case Py_LE:
      case Py_GT:
      case Py_GE:
        PyErr_SetString(PyExc_TypeError, (char *) "sidl_richcmp: Cannot order non-PETSc object");
    }
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  _return_value = PyObject_RichCompare(o1, o2, op);
  Py_DECREF(o1);
  Py_DECREF(o2);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyTypeObject SIDLPythonObjectType = {
  PyObject_HEAD_INIT(NULL)
  0,
  (char *) "PETSc.KSP",
  sizeof(struct PETSc_KSP_PythonObject),
  0,
  (destructor) sidl_dealloc,
  (printfunc) 0,
  (getattrfunc) 0,
  (setattrfunc) 0,
  (cmpfunc) sidl_cmp,
  (reprfunc) 0,
  (PyNumberMethods *) 0,
  (PySequenceMethods *) 0,
  (PyMappingMethods *) 0,
  (hashfunc) sidl_hash,
  (ternaryfunc) 0,
  (reprfunc) sidl_str,
  (getattrofunc) 0,
  (setattrofunc) 0,
  (PyBufferProcs *) 0,
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_WEAKREFS,
  (char *) "This is a Python wrapper for a SIDL object of type PETSc.KSP",
  (traverseproc) 0,
  (inquiry) 0,
  (richcmpfunc) sidl_richcmp,
  (int) &((struct PETSc_KSP_PythonObject *) 0)->weakRefList,
  (getiterfunc) 0,
  (iternextfunc) 0,
  _KSPObjectMethods,
  (struct PyMemberDef *) 0,
  (struct PyGetSetDef *) 0,
  0,
  0,
  (descrgetfunc) 0,
  (descrsetfunc) 0,
  0,
  (initproc) 0,
  (allocfunc) 0,
  (newfunc) PETSc_KSP_KSP,
  (freefunc) 0,
  (inquiry) 0};

void initKSP(void)
{
  PyObject *module = NULL;
  
  /* Setup the python type */
  if (PyType_Ready(&SIDLPythonObjectType) < 0)
  {
    return;
  }
  /* Initialize the module */
  module = Py_InitModule3((char *) "KSP", _KSPModuleMethods, (char *) "");
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Could not initialize module PETSc.KSP");
    return;
  }
  /* Add the type to the module and increment the ref count */
  Py_INCREF(&SIDLPythonObjectType);
  PyModule_AddObject(module, (char *) "KSP", (PyObject *) &SIDLPythonObjectType);
  /* Import Numeric */
  import_array();
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Cannot import Numeric Python array module");
    return;
  }
}
