#include <Python.h>

#include <Numeric/arrayobject.h>

#ifndef __PETSCPC_H
#include <petscpc.h>
#endif

enum ReferenceType {STOLEN_REF, WEAK_REF, NEW_REF};

struct PETSc_PC_PythonObject {
  PyObject_HEAD;
  PC ior;
  enum ReferenceType refType;
  PyObject *weakRefList;
};

/* Need to forward declare the type structure */
static PyTypeObject SIDLPythonObjectType;

static PyObject *sidl_new(PyTypeObject *type, PetscObject ior, enum ReferenceType refType)
{
  PyObject *return_value = NULL;
  
  if (ior)
  {
    struct PETSc_PC_PythonObject *self = (struct PETSc_PC_PythonObject *) type->tp_alloc(type, 0);
    
    self->weakRefList = NULL;
    if (self)
    {
      self->ior = (PC) ior;
      self->refType = refType;
      if (refType == NEW_REF)
      {
        PetscObjectReference(ior);
      }
      return_value = (PyObject *) self;
    }
    else
    {
      PyErr_SetString(PyExc_MemoryError, (char *) "Unable to allocate interface object");
    }
  }
  else
  {
    return_value = Py_None;
    Py_INCREF(return_value);
  }
  return return_value;
}

static PyObject *opaque__wrap_ior(void *ptr)
{
  if (ptr)
  {
    return PyCObject_FromVoidPtr(ptr, NULL);
  }
  else
  {
    Py_INCREF(Py_None);
    return Py_None;
  }
}

static PyObject *PETSc_PC__wrap_ior(PC ior)
{
  PyObject *module = PyImport_ImportModule((char *) "PETSc.PC");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "PC");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module PETSc.PC");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type PETSc.PC");
    return 0;
  }
  return sidl_new(type, (PetscObject) ior, STOLEN_REF);
}

static long PETSc_PetscViewer__extract_ior(struct PETSc_PC_PythonObject *obj, PetscViewer *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (PetscViewer) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "PetscViewer"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.PetscViewer");
    return 0;
  }
  return 1;
}

static long opaque__extract_ior(PyObject *obj, void **ptr)
{
  if (PyCObject_Check(obj))
  {
    *ptr = PyCObject_AsVoidPtr(obj);
    return 1;
  }
  /* Why in the hell is there no Python interface to CObject? */
  if (PyObject_HasAttrString(obj, "handle"))
  {
    *ptr = PyLong_AsVoidPtr(PyObject_GetAttrString(obj, "handle"));
    return 1;
  }
  if (obj == Py_None)
  {
    *ptr = 0;
    return 1;
  }
  return 0;
}

static long PETSc_PC__extract_ior(struct PETSc_PC_PythonObject *obj, PC *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (PC) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "PC"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.PC");
    return 0;
  }
  return 1;
}

static long PETSc_PetscObject__extract_ior(struct PETSc_PC_PythonObject *obj, PetscObject *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (PetscObject) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "PetscObject"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.PetscObject");
    return 0;
  }
  return 1;
}

static long PETSc_Vec__extract_ior(struct PETSc_PC_PythonObject *obj, Vec *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (Vec) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "Vec"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.Vec");
    return 0;
  }
  return 1;
}

static PyObject *PETSc_PetscObject_getType(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getType");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getType");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCGetType(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setType(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "typeName",
    NULL};
  long _okay = 0;
  const char *typeName = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setType");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setType");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &typeName);
  /* Copying the Python string to IOR controlled memory */
  typeName = typeName ? strcpy(malloc(strlen(typeName) + 1), typeName) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCSetType(_self_ior, typeName);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) typeName);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_getOptionsPrefix(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getOptionsPrefix");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getOptionsPrefix");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCGetOptionsPrefix(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setOptionsPrefix(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "prefix",
    NULL};
  long _okay = 0;
  const char *prefix = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setOptionsPrefix");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setOptionsPrefix");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &prefix);
  /* Copying the Python string to IOR controlled memory */
  prefix = prefix ? strcpy(malloc(strlen(prefix) + 1), prefix) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCSetOptionsPrefix(_self_ior, prefix);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) prefix);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_appendOptionsPrefix(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "prefix",
    NULL};
  long _okay = 0;
  const char *prefix = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for appendOptionsPrefix");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in appendOptionsPrefix");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &prefix);
  /* Copying the Python string to IOR controlled memory */
  prefix = prefix ? strcpy(malloc(strlen(prefix) + 1), prefix) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCAppendOptionsPrefix(_self_ior, prefix);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) prefix);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setFromOptions(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setFromOptions");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setFromOptions");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCSetFromOptions(_self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setUp(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setUp");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setUp");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCSetUp(_self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_view(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "viewer",
    NULL};
  long _okay = 0;
  PetscViewer viewer = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for view");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in view");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_PetscViewer__extract_ior, &viewer);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCView(_self_ior, viewer);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_getObject(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  void *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getObject");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getObject");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = ((struct PETSc_PC_PythonObject *) _self)->ior;
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) opaque__wrap_ior, _return_value);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setObject(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "object",
    NULL};
  long _okay = 0;
  void *object = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setObject");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setObject");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) opaque__extract_ior, &object);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  if ((((struct PETSc_PC_PythonObject *) _self)->refType != WEAK_REF) && (((struct PETSc_PC_PythonObject *) _self)->ior))
  {
    PCDestroy(((struct PETSc_PC_PythonObject *) _self)->ior);
  }
  ((struct PETSc_PC_PythonObject *) _self)->ior = object;
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getReferenceCount(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getReferenceCount");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getReferenceCount");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectGetReference((PetscObject) _self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_addReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for addReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in addReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectReference((PetscObject) _self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_deleteReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for deleteReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in deleteReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectDereference((PetscObject) _self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getClassName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  const char *_return_value = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getClassName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getClassName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = ((PetscObject) _self_ior)->class_name;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectGetName((PetscObject) _self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_setName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "name",
    NULL};
  long _okay = 0;
  const char *name = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &name);
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectSetName((PetscObject) _self_ior, name);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) name);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_isInstanceOf(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "typeName",
    NULL};
  long _okay = 0;
  const char *typeName = NULL;
  PetscTruth _return_value = 0;
  PetscTruth _local__return_value = 0;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isInstanceOf");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in isInstanceOf");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &typeName);
  /* Copying the Python string to IOR controlled memory */
  typeName = typeName ? strcpy(malloc(strlen(typeName) + 1), typeName) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = !strcmp(((PetscObject) _self_ior)->class_name, typeName);
  /* Convert PetscTruth to PetscTruth */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local__return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) typeName);
  return _pyReturn;
}

static PyObject *PETSc_PC_setUpOnBlocks(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setUpOnBlocks");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setUpOnBlocks");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCSetUpOnBlocks(_self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PC_apply(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "x",
    (char *) "y",
    NULL};
  long _okay = 0;
  Vec x = 0;
  Vec y = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for apply");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in apply");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&", _kwlist, (void *) PETSc_Vec__extract_ior, &x, (void *) PETSc_Vec__extract_ior, &y);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCApply(_self_ior, x, y);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PC_applySymmetricLeft(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "x",
    (char *) "y",
    NULL};
  long _okay = 0;
  Vec x = 0;
  Vec y = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for applySymmetricLeft");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in applySymmetricLeft");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&", _kwlist, (void *) PETSc_Vec__extract_ior, &x, (void *) PETSc_Vec__extract_ior, &y);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCApplySymmetricLeft(_self_ior, x, y);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PC_applySymmetricRight(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "x",
    (char *) "y",
    NULL};
  long _okay = 0;
  Vec x = 0;
  Vec y = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for applySymmetricRight");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in applySymmetricRight");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&", _kwlist, (void *) PETSc_Vec__extract_ior, &x, (void *) PETSc_Vec__extract_ior, &y);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCApplySymmetricRight(_self_ior, x, y);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PC_applyBAorAB(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "side",
    (char *) "x",
    (char *) "y",
    (char *) "work",
    NULL};
  long _okay = 0;
  PCSide side = 0;
  long _local_side = 0;
  Vec x = 0;
  Vec y = 0;
  Vec work = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for applyBAorAB");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in applyBAorAB");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "lO&O&O&", _kwlist, &_local_side, (void *) PETSc_Vec__extract_ior, &x, (void *) PETSc_Vec__extract_ior, &y, (void *) PETSc_Vec__extract_ior, &work);
  /* Convert long to PCSide */
  side = _local_side;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCApplyBAorAB(_self_ior, side, x, y, work);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PC_applyTranspose(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "x",
    (char *) "y",
    NULL};
  long _okay = 0;
  Vec x = 0;
  Vec y = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for applyTranspose");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in applyTranspose");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&", _kwlist, (void *) PETSc_Vec__extract_ior, &x, (void *) PETSc_Vec__extract_ior, &y);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCApplyTranspose(_self_ior, x, y);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PC_hasApplyTranspose(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  PetscTruth _return_value = 0;
  PetscTruth _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for hasApplyTranspose");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in hasApplyTranspose");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCHasApplyTranspose(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert PetscTruth to PetscTruth */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local__return_value);
  return _pyReturn;
}

static PyObject *PETSc_PC_applyBAorABTranspose(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "side",
    (char *) "x",
    (char *) "y",
    (char *) "work",
    NULL};
  long _okay = 0;
  PCSide side = 0;
  long _local_side = 0;
  Vec x = 0;
  Vec y = 0;
  Vec work = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for applyBAorABTranspose");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in applyBAorABTranspose");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "lO&O&O&", _kwlist, &_local_side, (void *) PETSc_Vec__extract_ior, &x, (void *) PETSc_Vec__extract_ior, &y, (void *) PETSc_Vec__extract_ior, &work);
  /* Convert long to PCSide */
  side = _local_side;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCApplyBAorABTranspose(_self_ior, side, x, y, work);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PC_applyRichardson(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "x",
    (char *) "y",
    (char *) "work",
    (char *) "relativeTol",
    (char *) "absoluteTol",
    (char *) "divergenceTol",
    (char *) "numIterations",
    NULL};
  long _okay = 0;
  Vec x = 0;
  Vec y = 0;
  Vec work = 0;
  double relativeTol = 0.0;
  double absoluteTol = 0.0;
  double divergenceTol = 0.0;
  int32_t numIterations = 0;
  long _local_numIterations = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for applyRichardson");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in applyRichardson");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&O&dddl", _kwlist, (void *) PETSc_Vec__extract_ior, &x, (void *) PETSc_Vec__extract_ior, &y, (void *) PETSc_Vec__extract_ior, &work, &relativeTol, &absoluteTol, &divergenceTol, &_local_numIterations);
  /* Convert long to int32_t */
  numIterations = _local_numIterations;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCApplyRichardson(_self_ior, x, y, work, relativeTol, absoluteTol, divergenceTol, numIterations);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PC_applyRichardsonExists(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  PetscTruth richardsonExists = 0;
  PetscTruth _local_richardsonExists = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for applyRichardsonExists");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in applyRichardsonExists");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PCApplyRichardsonExists(_self_ior, &richardsonExists);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert PetscTruth to PetscTruth */
  _local_richardsonExists = richardsonExists;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local_richardsonExists);
  return _pyReturn;
}

static PyObject *PETSc_PC_PC(PyObject *_type, PyObject *_args, PyObject *_kwdict)
{
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  long _okay = 0;
  PC obj = 0;
  PetscTruth created = 0;
  PC _return_value = 0;
  PyObject *_pyReturn = NULL;
  
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "|O&", _kwlist, (void *) PETSc_PC__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  if (obj)
  {
    created = 0;
  }
  else
  {
    created = 1;
    PCCreate(PETSC_COMM_WORLD, &obj);
  }
  _return_value = obj;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) PETSc_PC__wrap_ior, _return_value);
  if (_return_value)
  {
    PetscObjectReference(((PetscObject) _return_value));
  }
  if (created)
  {
    PetscObjectDereference((PetscObject) obj);
  }
  return _pyReturn;
}

static PyObject *isSame(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  /* Retrieve the IOR pointer */
  PC _self_ior = ((struct PETSc_PC_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  long _okay = 0;
  PetscObject obj = 0;
  PetscTruth _return_value = 0;
  PetscTruth _local__return_value = 0;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isSame");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in isSame");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_PetscObject__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = (_self_ior == (PC) obj);
  /* Convert PetscTruth to PetscTruth */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local__return_value);
  return _pyReturn;
}

static struct PyMethodDef _PCObjectMethods[] = {
  {(char *) "getType", (PyCFunction) PETSc_PetscObject_getType, (METH_VARARGS | METH_KEYWORDS), (char *) "getType()\n\
RETURNS (return string _return_value)\n\
 Get the implementation type"},
  {(char *) "setType", (PyCFunction) PETSc_PetscObject_setType, (METH_VARARGS | METH_KEYWORDS), (char *) "setType(in string typeName)\n\
RETURNS None\n\
\n\
 Set the implementation type"},
  {(char *) "getOptionsPrefix", (PyCFunction) PETSc_PetscObject_getOptionsPrefix, (METH_VARARGS | METH_KEYWORDS), (char *) "getOptionsPrefix()\n\
RETURNS (return string _return_value)\n\
 Get the prefix appended to options directed to this object"},
  {(char *) "setOptionsPrefix", (PyCFunction) PETSc_PetscObject_setOptionsPrefix, (METH_VARARGS | METH_KEYWORDS), (char *) "setOptionsPrefix(in string prefix)\n\
RETURNS None\n\
\n\
 Set the prefix appended to options directed to this object"},
  {(char *) "appendOptionsPrefix", (PyCFunction) PETSc_PetscObject_appendOptionsPrefix, (METH_VARARGS | METH_KEYWORDS), (char *) "appendOptionsPrefix(in string prefix)\n\
RETURNS None\n\
\n\
 Enlarge the prefix appended to options directed to this object"},
  {(char *) "setFromOptions", (PyCFunction) PETSc_PetscObject_setFromOptions, (METH_VARARGS | METH_KEYWORDS), (char *) "setFromOptions()\n\
RETURNS None\n\
\n\
 Configure the object using the PETSc options database"},
  {(char *) "setUp", (PyCFunction) PETSc_PetscObject_setUp, (METH_VARARGS | METH_KEYWORDS), (char *) "setUp()\n\
RETURNS None\n\
\n\
 Initiate the object setup phase"},
  {(char *) "view", (PyCFunction) PETSc_PetscObject_view, (METH_VARARGS | METH_KEYWORDS), (char *) "view(in PETSc.PetscViewer viewer)\n\
RETURNS None\n\
\n\
 Output a representation of the object using "},
  {(char *) "getObject", (PyCFunction) PETSc_PetscObject_getObject, (METH_VARARGS | METH_KEYWORDS), (char *) "getObject()\n\
RETURNS (return opaque _return_value)\n\
 Get the PETSc object pointer"},
  {(char *) "setObject", (PyCFunction) PETSc_PetscObject_setObject, (METH_VARARGS | METH_KEYWORDS), (char *) "setObject(in opaque object)\n\
RETURNS None\n\
\n\
 Set the PETSc object pointer"},
  {(char *) "getReferenceCount", (PyCFunction) ASE_BaseInterface_getReferenceCount, (METH_VARARGS | METH_KEYWORDS), (char *) "getReferenceCount()\n\
RETURNS (return int _return_value)\n\
 Retrieve the intrinsic reference count of the underlying object"},
  {(char *) "addReference", (PyCFunction) ASE_BaseInterface_addReference, (METH_VARARGS | METH_KEYWORDS), (char *) "addReference()\n\
RETURNS None\n\
\n\
 Add one to the intrinsic reference count of the underlying object - Objects continue to exist as long as the reference count is positive - Clients should call this method whenever they create another ongoing reference to an object or interface"},
  {(char *) "deleteReference", (PyCFunction) ASE_BaseInterface_deleteReference, (METH_VARARGS | METH_KEYWORDS), (char *) "deleteReference()\n\
RETURNS None\n\
\n\
 Decrease by one the intrinsic reference count of the underlying object - Delete the object if the reference count is non-positive - Clients should call this method whenever they remove a reference to an object or interface"},
  {(char *) "getClassName", (PyCFunction) ASE_BaseInterface_getClassName, (METH_VARARGS | METH_KEYWORDS), (char *) "getClassName()\n\
RETURNS (return string _return_value)\n\
 Return the type name of the underlying class object"},
  {(char *) "getName", (PyCFunction) ASE_BaseInterface_getName, (METH_VARARGS | METH_KEYWORDS), (char *) "getName()\n\
RETURNS (return string _return_value)\n\
 Retrieve the object name"},
  {(char *) "setName", (PyCFunction) ASE_BaseInterface_setName, (METH_VARARGS | METH_KEYWORDS), (char *) "setName(in string name)\n\
RETURNS None\n\
\n\
 Set the object name"},
  {(char *) "isInstanceOf", (PyCFunction) ASE_BaseInterface_isInstanceOf, (METH_VARARGS | METH_KEYWORDS), (char *) "isInstanceOf(in string typeName)\n\
RETURNS (return bool _return_value)\n\
 Return <code>true</code> if this object is an instance of the specified type - This routine will return <code>true</code> if and only if a cast to the specified type would succeed"},
  {(char *) "setUpOnBlocks", (PyCFunction) PETSc_PC_setUpOnBlocks, (METH_VARARGS | METH_KEYWORDS), (char *) "setUpOnBlocks()\n\
RETURNS None\n\
\n\
 Setup local preconditioner blocks"},
  {(char *) "apply", (PyCFunction) PETSc_PC_apply, (METH_VARARGS | METH_KEYWORDS), (char *) "apply(in PETSc.Vec x, in PETSc.Vec y)\n\
RETURNS None\n\
\n\
 Apply the preconditioner M^{-1} x = y"},
  {(char *) "applySymmetricLeft", (PyCFunction) PETSc_PC_applySymmetricLeft, (METH_VARARGS | METH_KEYWORDS), (char *) "applySymmetricLeft(in PETSc.Vec x, in PETSc.Vec y)\n\
RETURNS None\n\
\n\
 Apply the preconditioner M_L^{-1/2} x = y"},
  {(char *) "applySymmetricRight", (PyCFunction) PETSc_PC_applySymmetricRight, (METH_VARARGS | METH_KEYWORDS), (char *) "applySymmetricRight(in PETSc.Vec x, in PETSc.Vec y)\n\
RETURNS None\n\
\n\
 Apply the preconditioner M_R^{-1/2} x = y"},
  {(char *) "applyBAorAB", (PyCFunction) PETSc_PC_applyBAorAB, (METH_VARARGS | METH_KEYWORDS), (char *) "applyBAorAB(in PETSc.PCSide side, in PETSc.Vec x, in PETSc.Vec y, in PETSc.Vec work)\n\
RETURNS None\n\
\n\
 Apply M^{-1} to x and store the result in y"},
  {(char *) "applyTranspose", (PyCFunction) PETSc_PC_applyTranspose, (METH_VARARGS | METH_KEYWORDS), (char *) "applyTranspose(in PETSc.Vec x, in PETSc.Vec y)\n\
RETURNS None\n\
\n\
 Apply the preconditioner M^{-T} x = y"},
  {(char *) "hasApplyTranspose", (PyCFunction) PETSc_PC_hasApplyTranspose, (METH_VARARGS | METH_KEYWORDS), (char *) "hasApplyTranspose()\n\
RETURNS (return bool _return_value)\n\
 Determine whether M^{-T} is available"},
  {(char *) "applyBAorABTranspose", (PyCFunction) PETSc_PC_applyBAorABTranspose, (METH_VARARGS | METH_KEYWORDS), (char *) "applyBAorABTranspose(in PETSc.PCSide side, in PETSc.Vec x, in PETSc.Vec y, in PETSc.Vec work)\n\
RETURNS None\n\
\n\
 Apply M^{-T} to x and store the result in y"},
  {(char *) "applyRichardson", (PyCFunction) PETSc_PC_applyRichardson, (METH_VARARGS | METH_KEYWORDS), (char *) "applyRichardson(in PETSc.Vec x, in PETSc.Vec y, in PETSc.Vec work, in double relativeTol, in double absoluteTol, in double divergenceTol, in int numIterations)\n\
RETURNS None\n\
\n\
 Apply Richardson exstrapolation"},
  {(char *) "applyRichardsonExists", (PyCFunction) PETSc_PC_applyRichardsonExists, (METH_VARARGS | METH_KEYWORDS), (char *) "applyRichardsonExists(out bool richardsonExists)\n\
RETURNS (out bool richardsonExists)\n\
 Determine whether Richardson extrapolation is available"},
  {(char *) "isSame", (PyCFunction) isSame, (METH_VARARGS | METH_KEYWORDS), (char *) "isSame(in PETSc.PetscObject obj)\n\
RETURNS (return bool _return_value)\n\
"},
  {NULL, NULL}};

static struct PyMethodDef _PCModuleMethods[] = {{NULL, NULL}};

static void sidl_dealloc(struct PETSc_PC_PythonObject *self)
{
  if (self->weakRefList)
  {
    PyObject_ClearWeakRefs((PyObject *) self);
  }
  if ((self->refType != WEAK_REF) && (self->ior))
  {
    PCDestroy(self->ior);
  }
  self->ob_type->tp_free(self);
}

static int sidl_cmp(struct PETSc_PC_PythonObject *self, PyObject *arg)
{
  PC ior1 = self->ior;
  PetscObject ior2 = (PetscObject) ((struct PETSc_PC_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (arg->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for sidl_cmp");
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  _return_value = PyObject_Compare(o1, o2);
  Py_DECREF(o1);
  Py_DECREF(o2);
  return _return_value;
}

static long sidl_hash(struct PETSc_PC_PythonObject *self)
{
  PC ior = self->ior;
  PyObject *_hash_arg = NULL;
  long _return_value = 0;
  
  if (!ior)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_hash()");
    return _return_value;
  }
  _hash_arg = Py_BuildValue((char *) "i", ior);
  if (!_hash_arg)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not hash IOR pointer in sidl_hash()");
    return _return_value;
  }
  _return_value = PyObject_Hash(_hash_arg);
  Py_DECREF(_hash_arg);
  return _return_value;
}

static PyObject *sidl_str(struct PETSc_PC_PythonObject *self)
{
  const char *_str = NULL;
  PyObject *_return_value = NULL;
  PC ior = self->ior;
  
  PCGetType(ior, (PCType *) &_str);
  _return_value = PyString_FromString(_str);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyObject *sidl_richcmp(struct PETSc_PC_PythonObject *self, PyObject *arg, int op)
{
  PC ior1 = self->ior;
  PetscObject ior2 = (PetscObject) ((struct PETSc_PC_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  PyObject *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (!PyObject_HasAttrString(arg, (char *) "getClassName"))
  {
    switch(op) {
      case Py_EQ:
        _return_value = Py_False;
        Py_INCREF(_return_value);
        break;
      case Py_NE:
        _return_value = Py_True;
        Py_INCREF(_return_value);
        break;
      case Py_LT:
      case Py_LE:
      case Py_GT:
      case Py_GE:
        PyErr_SetString(PyExc_TypeError, (char *) "sidl_richcmp: Cannot order non-PETSc object");
    }
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  _return_value = PyObject_RichCompare(o1, o2, op);
  Py_DECREF(o1);
  Py_DECREF(o2);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyTypeObject SIDLPythonObjectType = {
  PyObject_HEAD_INIT(NULL)
  0,
  (char *) "PETSc.PC",
  sizeof(struct PETSc_PC_PythonObject),
  0,
  (destructor) sidl_dealloc,
  (printfunc) 0,
  (getattrfunc) 0,
  (setattrfunc) 0,
  (cmpfunc) sidl_cmp,
  (reprfunc) 0,
  (PyNumberMethods *) 0,
  (PySequenceMethods *) 0,
  (PyMappingMethods *) 0,
  (hashfunc) sidl_hash,
  (ternaryfunc) 0,
  (reprfunc) sidl_str,
  (getattrofunc) 0,
  (setattrofunc) 0,
  (PyBufferProcs *) 0,
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_WEAKREFS,
  (char *) "This is a Python wrapper for a SIDL object of type PETSc.PC",
  (traverseproc) 0,
  (inquiry) 0,
  (richcmpfunc) sidl_richcmp,
  (int) &((struct PETSc_PC_PythonObject *) 0)->weakRefList,
  (getiterfunc) 0,
  (iternextfunc) 0,
  _PCObjectMethods,
  (struct PyMemberDef *) 0,
  (struct PyGetSetDef *) 0,
  0,
  0,
  (descrgetfunc) 0,
  (descrsetfunc) 0,
  0,
  (initproc) 0,
  (allocfunc) 0,
  (newfunc) PETSc_PC_PC,
  (freefunc) 0,
  (inquiry) 0};

void initPC(void)
{
  PyObject *module = NULL;
  
  /* Setup the python type */
  if (PyType_Ready(&SIDLPythonObjectType) < 0)
  {
    return;
  }
  /* Initialize the module */
  module = Py_InitModule3((char *) "PC", _PCModuleMethods, (char *) "");
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Could not initialize module PETSc.PC");
    return;
  }
  /* Add the type to the module and increment the ref count */
  Py_INCREF(&SIDLPythonObjectType);
  PyModule_AddObject(module, (char *) "PC", (PyObject *) &SIDLPythonObjectType);
  /* Import Numeric */
  import_array();
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Cannot import Numeric Python array module");
    return;
  }
}
