#include <Python.h>

#include <Numeric/arrayobject.h>

#ifndef __PETSC_H
#include <petsc.h>
#endif

enum ReferenceType {STOLEN_REF, WEAK_REF, NEW_REF};

struct PETSc_PetscOptions_PythonObject {
  PyObject_HEAD;
  PetscObject ior;
  enum ReferenceType refType;
  PyObject *weakRefList;
};

/* Need to forward declare the type structure */
static PyTypeObject SIDLPythonObjectType;

static PyObject *sidl_new(PyTypeObject *type, PetscObject ior, enum ReferenceType refType)
{
  PyObject *return_value = NULL;
  
  if (ior)
  {
    struct PETSc_PetscOptions_PythonObject *self = (struct PETSc_PetscOptions_PythonObject *) type->tp_alloc(type, 0);
    
    self->weakRefList = NULL;
    if (self)
    {
      self->ior = (PetscObject) ior;
      self->refType = refType;
      if (refType == NEW_REF)
      {
        PetscObjectReference(ior);
      }
      return_value = (PyObject *) self;
    }
    else
    {
      PyErr_SetString(PyExc_MemoryError, (char *) "Unable to allocate interface object");
    }
  }
  else
  {
    return_value = Py_None;
    Py_INCREF(return_value);
  }
  return return_value;
}

static PyObject *opaque__wrap_ior(void *ptr)
{
  if (ptr)
  {
    return PyCObject_FromVoidPtr(ptr, NULL);
  }
  else
  {
    Py_INCREF(Py_None);
    return Py_None;
  }
}

static PyObject *PETSc_PetscOptions__wrap_ior(PetscObject ior)
{
  PyObject *module = PyImport_ImportModule((char *) "PETSc.PetscOptions");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "PetscOptions");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module PETSc.PetscOptions");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type PETSc.PetscOptions");
    return 0;
  }
  return sidl_new(type, (PetscObject) ior, STOLEN_REF);
}

static long PETSc_PetscViewer__extract_ior(struct PETSc_PetscOptions_PythonObject *obj, PetscViewer *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (PetscViewer) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "PetscViewer"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.PetscViewer");
    return 0;
  }
  return 1;
}

static long opaque__extract_ior(PyObject *obj, void **ptr)
{
  if (PyCObject_Check(obj))
  {
    *ptr = PyCObject_AsVoidPtr(obj);
    return 1;
  }
  /* Why in the hell is there no Python interface to CObject? */
  if (PyObject_HasAttrString(obj, "handle"))
  {
    *ptr = PyLong_AsVoidPtr(PyObject_GetAttrString(obj, "handle"));
    return 1;
  }
  if (obj == Py_None)
  {
    *ptr = 0;
    return 1;
  }
  return 0;
}

static long PETSc_PetscOptions__extract_ior(struct PETSc_PetscOptions_PythonObject *obj, PetscObject *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (PetscObject) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "PetscOptions"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.PetscOptions");
    return 0;
  }
  return 1;
}

static long PETSc_PetscObject__extract_ior(struct PETSc_PetscOptions_PythonObject *obj, PetscObject *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (PetscObject) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "PetscObject"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.PetscObject");
    return 0;
  }
  return 1;
}

static PyObject *PETSc_PetscObject_getType(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getType");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getType");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectGetType(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setType(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "typeName",
    NULL};
  long _okay = 0;
  const char *typeName = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setType");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setType");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &typeName);
  /* Copying the Python string to IOR controlled memory */
  typeName = typeName ? strcpy(malloc(strlen(typeName) + 1), typeName) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectSetType(_self_ior, typeName);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) typeName);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_getOptionsPrefix(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getOptionsPrefix");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getOptionsPrefix");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectGetOptionsPrefix(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setOptionsPrefix(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "prefix",
    NULL};
  long _okay = 0;
  const char *prefix = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setOptionsPrefix");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setOptionsPrefix");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &prefix);
  /* Copying the Python string to IOR controlled memory */
  prefix = prefix ? strcpy(malloc(strlen(prefix) + 1), prefix) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectSetOptionsPrefix(_self_ior, prefix);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) prefix);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_appendOptionsPrefix(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "prefix",
    NULL};
  long _okay = 0;
  const char *prefix = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for appendOptionsPrefix");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in appendOptionsPrefix");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &prefix);
  /* Copying the Python string to IOR controlled memory */
  prefix = prefix ? strcpy(malloc(strlen(prefix) + 1), prefix) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectAppendOptionsPrefix(_self_ior, prefix);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) prefix);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setFromOptions(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setFromOptions");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setFromOptions");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectSetFromOptions(_self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setUp(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setUp");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setUp");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectSetUp(_self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_view(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "viewer",
    NULL};
  long _okay = 0;
  PetscViewer viewer = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for view");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in view");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_PetscViewer__extract_ior, &viewer);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectView(_self_ior, viewer);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_getObject(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  void *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getObject");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getObject");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) opaque__wrap_ior, _return_value);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setObject(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "object",
    NULL};
  long _okay = 0;
  void *object = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setObject");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setObject");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) opaque__extract_ior, &object);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  if ((((struct PETSc_PetscOptions_PythonObject *) _self)->refType != WEAK_REF) && (((struct PETSc_PetscOptions_PythonObject *) _self)->ior))
  {
    PetscObjectDestroy(((struct PETSc_PetscOptions_PythonObject *) _self)->ior);
  }
  ((struct PETSc_PetscOptions_PythonObject *) _self)->ior = object;
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getReferenceCount(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getReferenceCount");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getReferenceCount");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectGetReference((PetscObject) _self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_addReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for addReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in addReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectReference((PetscObject) _self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_deleteReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for deleteReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in deleteReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectDereference((PetscObject) _self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getClassName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  const char *_return_value = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getClassName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getClassName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = ((PetscObject) _self_ior)->class_name;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectGetName((PetscObject) _self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_setName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "name",
    NULL};
  long _okay = 0;
  const char *name = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &name);
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectSetName((PetscObject) _self_ior, name);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) name);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_isInstanceOf(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "typeName",
    NULL};
  long _okay = 0;
  const char *typeName = NULL;
  PetscTruth _return_value = 0;
  PetscTruth _local__return_value = 0;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isInstanceOf");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in isInstanceOf");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &typeName);
  /* Copying the Python string to IOR controlled memory */
  typeName = typeName ? strcpy(malloc(strlen(typeName) + 1), typeName) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = !strcmp(((PetscObject) _self_ior)->class_name, typeName);
  /* Convert PetscTruth to PetscTruth */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local__return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) typeName);
  return _pyReturn;
}

static PyObject *PETSc_PetscOptions_hasName(PyObject *_ignored, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  static char *_kwlist[] = {
    (char *) "pre",
    (char *) "name",
    NULL};
  long _okay = 0;
  const char *pre = NULL;
  const char *name = NULL;
  int ierr = 0;
  PetscTruth _return_value = 0;
  PetscTruth _local__return_value = 0;
  
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "zz", _kwlist, &pre, &name);
  /* Copying the Python string to IOR controlled memory */
  pre = pre ? strcpy(malloc(strlen(pre) + 1), pre) : NULL;
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscOptionsHasName(pre, name, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert PetscTruth to PetscTruth */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local__return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) pre);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) name);
  return _pyReturn;
}

static PyObject *PETSc_PetscOptions_setValue(PyObject *_ignored, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  static char *_kwlist[] = {
    (char *) "name",
    (char *) "value",
    NULL};
  long _okay = 0;
  const char *name = NULL;
  const char *value = NULL;
  int ierr = 0;
  
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "zz", _kwlist, &name, &value);
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  /* Copying the Python string to IOR controlled memory */
  value = value ? strcpy(malloc(strlen(value) + 1), value) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscOptionsSetValue(name, value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) name);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) value);
  return _pyReturn;
}

static PyObject *PETSc_PetscOptions_clearValue(PyObject *_ignored, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  static char *_kwlist[] = {
    (char *) "name",
    NULL};
  long _okay = 0;
  const char *name = NULL;
  int ierr = 0;
  
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &name);
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscOptionsClearValue(name);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) name);
  return _pyReturn;
}

static PyObject *PETSc_PetscOptions_getInt(PyObject *_ignored, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  static char *_kwlist[] = {
    (char *) "pre",
    (char *) "name",
    NULL};
  long _okay = 0;
  const char *pre = NULL;
  const char *name = NULL;
  int ierr = 0;
  int32_t value = 0;
  long _local_value = 0;
  PetscTruth flag = 0;
  PetscTruth _local_flag = 0;
  
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "zz", _kwlist, &pre, &name);
  /* Copying the Python string to IOR controlled memory */
  pre = pre ? strcpy(malloc(strlen(pre) + 1), pre) : NULL;
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscOptionsGetInt(pre, name, &value, &flag);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local_value = value;
  /* Convert PetscTruth to PetscTruth */
  _local_flag = flag;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "(li)", _local_value, _local_flag);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) pre);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) name);
  return _pyReturn;
}

static PyObject *PETSc_PetscOptions_getReal(PyObject *_ignored, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  static char *_kwlist[] = {
    (char *) "pre",
    (char *) "name",
    NULL};
  long _okay = 0;
  const char *pre = NULL;
  const char *name = NULL;
  int ierr = 0;
  double value = 0.0;
  PetscTruth flag = 0;
  PetscTruth _local_flag = 0;
  
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "zz", _kwlist, &pre, &name);
  /* Copying the Python string to IOR controlled memory */
  pre = pre ? strcpy(malloc(strlen(pre) + 1), pre) : NULL;
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscOptionsGetReal(pre, name, &value, &flag);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert PetscTruth to PetscTruth */
  _local_flag = flag;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "(di)", value, _local_flag);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) pre);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) name);
  return _pyReturn;
}

static PyObject *PETSc_PetscOptions_getScalar(PyObject *_ignored, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  static char *_kwlist[] = {
    (char *) "pre",
    (char *) "name",
    NULL};
  long _okay = 0;
  const char *pre = NULL;
  const char *name = NULL;
  int ierr = 0;
  double value = 0.0;
  PetscTruth flag = 0;
  PetscTruth _local_flag = 0;
  
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "zz", _kwlist, &pre, &name);
  /* Copying the Python string to IOR controlled memory */
  pre = pre ? strcpy(malloc(strlen(pre) + 1), pre) : NULL;
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscOptionsGetScalar(pre, name, &value, &flag);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert PetscTruth to PetscTruth */
  _local_flag = flag;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "(di)", value, _local_flag);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) pre);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) name);
  return _pyReturn;
}

static PyObject *PETSc_PetscOptions_getTruth(PyObject *_ignored, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  static char *_kwlist[] = {
    (char *) "pre",
    (char *) "name",
    NULL};
  long _okay = 0;
  const char *pre = NULL;
  const char *name = NULL;
  int ierr = 0;
  PetscTruth value = 0;
  PetscTruth _local_value = 0;
  PetscTruth flag = 0;
  PetscTruth _local_flag = 0;
  
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "zz", _kwlist, &pre, &name);
  /* Copying the Python string to IOR controlled memory */
  pre = pre ? strcpy(malloc(strlen(pre) + 1), pre) : NULL;
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscOptionsGetTruth(pre, name, &value, &flag);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert PetscTruth to PetscTruth */
  _local_value = value;
  /* Convert PetscTruth to PetscTruth */
  _local_flag = flag;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "(ii)", _local_value, _local_flag);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) pre);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) name);
  return _pyReturn;
}

static PyObject *PETSc_PetscOptions_PetscOptions(PyObject *_type, PyObject *_args, PyObject *_kwdict)
{
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  long _okay = 0;
  PetscObject obj = 0;
  PetscTruth created = 0;
  PetscObject _return_value = 0;
  PyObject *_pyReturn = NULL;
  
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "|O&", _kwlist, (void *) PETSc_PetscOptions__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  if (obj)
  {
    created = 0;
  }
  else
  {
    created = 1;
    PetscObjectCreate(PETSC_COMM_WORLD, &obj);
  }
  _return_value = obj;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) PETSc_PetscOptions__wrap_ior, _return_value);
  if (_return_value)
  {
    PetscObjectReference(((PetscObject) _return_value));
  }
  if (created)
  {
    PetscObjectDereference((PetscObject) obj);
  }
  return _pyReturn;
}

static PyObject *isSame(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  /* Retrieve the IOR pointer */
  PetscObject _self_ior = ((struct PETSc_PetscOptions_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  long _okay = 0;
  PetscObject obj = 0;
  PetscTruth _return_value = 0;
  PetscTruth _local__return_value = 0;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isSame");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in isSame");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_PetscObject__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = (_self_ior == (PetscObject) obj);
  /* Convert PetscTruth to PetscTruth */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local__return_value);
  return _pyReturn;
}

static struct PyMethodDef _PetscOptionsObjectMethods[] = {
  {(char *) "getType", (PyCFunction) PETSc_PetscObject_getType, (METH_VARARGS | METH_KEYWORDS), (char *) "getType()\n\
RETURNS (return string _return_value)\n\
 Get the implementation type"},
  {(char *) "setType", (PyCFunction) PETSc_PetscObject_setType, (METH_VARARGS | METH_KEYWORDS), (char *) "setType(in string typeName)\n\
RETURNS None\n\
\n\
 Set the implementation type"},
  {(char *) "getOptionsPrefix", (PyCFunction) PETSc_PetscObject_getOptionsPrefix, (METH_VARARGS | METH_KEYWORDS), (char *) "getOptionsPrefix()\n\
RETURNS (return string _return_value)\n\
 Get the prefix appended to options directed to this object"},
  {(char *) "setOptionsPrefix", (PyCFunction) PETSc_PetscObject_setOptionsPrefix, (METH_VARARGS | METH_KEYWORDS), (char *) "setOptionsPrefix(in string prefix)\n\
RETURNS None\n\
\n\
 Set the prefix appended to options directed to this object"},
  {(char *) "appendOptionsPrefix", (PyCFunction) PETSc_PetscObject_appendOptionsPrefix, (METH_VARARGS | METH_KEYWORDS), (char *) "appendOptionsPrefix(in string prefix)\n\
RETURNS None\n\
\n\
 Enlarge the prefix appended to options directed to this object"},
  {(char *) "setFromOptions", (PyCFunction) PETSc_PetscObject_setFromOptions, (METH_VARARGS | METH_KEYWORDS), (char *) "setFromOptions()\n\
RETURNS None\n\
\n\
 Configure the object using the PETSc options database"},
  {(char *) "setUp", (PyCFunction) PETSc_PetscObject_setUp, (METH_VARARGS | METH_KEYWORDS), (char *) "setUp()\n\
RETURNS None\n\
\n\
 Initiate the object setup phase"},
  {(char *) "view", (PyCFunction) PETSc_PetscObject_view, (METH_VARARGS | METH_KEYWORDS), (char *) "view(in PETSc.PetscViewer viewer)\n\
RETURNS None\n\
\n\
 Output a representation of the object using "},
  {(char *) "getObject", (PyCFunction) PETSc_PetscObject_getObject, (METH_VARARGS | METH_KEYWORDS), (char *) "getObject()\n\
RETURNS (return opaque _return_value)\n\
 Get the PETSc object pointer"},
  {(char *) "setObject", (PyCFunction) PETSc_PetscObject_setObject, (METH_VARARGS | METH_KEYWORDS), (char *) "setObject(in opaque object)\n\
RETURNS None\n\
\n\
 Set the PETSc object pointer"},
  {(char *) "getReferenceCount", (PyCFunction) ASE_BaseInterface_getReferenceCount, (METH_VARARGS | METH_KEYWORDS), (char *) "getReferenceCount()\n\
RETURNS (return int _return_value)\n\
 Retrieve the intrinsic reference count of the underlying object"},
  {(char *) "addReference", (PyCFunction) ASE_BaseInterface_addReference, (METH_VARARGS | METH_KEYWORDS), (char *) "addReference()\n\
RETURNS None\n\
\n\
 Add one to the intrinsic reference count of the underlying object - Objects continue to exist as long as the reference count is positive - Clients should call this method whenever they create another ongoing reference to an object or interface"},
  {(char *) "deleteReference", (PyCFunction) ASE_BaseInterface_deleteReference, (METH_VARARGS | METH_KEYWORDS), (char *) "deleteReference()\n\
RETURNS None\n\
\n\
 Decrease by one the intrinsic reference count of the underlying object - Delete the object if the reference count is non-positive - Clients should call this method whenever they remove a reference to an object or interface"},
  {(char *) "getClassName", (PyCFunction) ASE_BaseInterface_getClassName, (METH_VARARGS | METH_KEYWORDS), (char *) "getClassName()\n\
RETURNS (return string _return_value)\n\
 Return the type name of the underlying class object"},
  {(char *) "getName", (PyCFunction) ASE_BaseInterface_getName, (METH_VARARGS | METH_KEYWORDS), (char *) "getName()\n\
RETURNS (return string _return_value)\n\
 Retrieve the object name"},
  {(char *) "setName", (PyCFunction) ASE_BaseInterface_setName, (METH_VARARGS | METH_KEYWORDS), (char *) "setName(in string name)\n\
RETURNS None\n\
\n\
 Set the object name"},
  {(char *) "isInstanceOf", (PyCFunction) ASE_BaseInterface_isInstanceOf, (METH_VARARGS | METH_KEYWORDS), (char *) "isInstanceOf(in string typeName)\n\
RETURNS (return bool _return_value)\n\
 Return <code>true</code> if this object is an instance of the specified type - This routine will return <code>true</code> if and only if a cast to the specified type would succeed"},
  {(char *) "hasName", (PyCFunction) PETSc_PetscOptions_hasName, (METH_VARARGS | METH_KEYWORDS | METH_STATIC), (char *) "hasName(in string pre, in string name)\n\
RETURNS (return bool _return_value)\n\
 Determines whether a certain option is given in the database"},
  {(char *) "setValue", (PyCFunction) PETSc_PetscOptions_setValue, (METH_VARARGS | METH_KEYWORDS | METH_STATIC), (char *) "setValue(in string name, in string value)\n\
RETURNS None\n\
\n\
 Sets an option name-value pair in the options database, overriding whatever is already present"},
  {(char *) "clearValue", (PyCFunction) PETSc_PetscOptions_clearValue, (METH_VARARGS | METH_KEYWORDS | METH_STATIC), (char *) "clearValue(in string name)\n\
RETURNS None\n\
\n\
 Clears an option name-value pair in the options database, overriding whatever is already present"},
  {(char *) "getInt", (PyCFunction) PETSc_PetscOptions_getInt, (METH_VARARGS | METH_KEYWORDS | METH_STATIC), (char *) "getInt(in string pre, in string name, out int value, out bool flag)\n\
RETURNS (out int value, out bool flag)\n\
"},
  {(char *) "getReal", (PyCFunction) PETSc_PetscOptions_getReal, (METH_VARARGS | METH_KEYWORDS | METH_STATIC), (char *) "getReal(in string pre, in string name, out double value, out bool flag)\n\
RETURNS (out double value, out bool flag)\n\
"},
  {(char *) "getScalar", (PyCFunction) PETSc_PetscOptions_getScalar, (METH_VARARGS | METH_KEYWORDS | METH_STATIC), (char *) "getScalar(in string pre, in string name, out double value, out bool flag)\n\
RETURNS (out double value, out bool flag)\n\
"},
  {(char *) "getTruth", (PyCFunction) PETSc_PetscOptions_getTruth, (METH_VARARGS | METH_KEYWORDS | METH_STATIC), (char *) "getTruth(in string pre, in string name, out bool value, out bool flag)\n\
RETURNS (out bool value, out bool flag)\n\
"},
  {(char *) "isSame", (PyCFunction) isSame, (METH_VARARGS | METH_KEYWORDS), (char *) "isSame(in PETSc.PetscObject obj)\n\
RETURNS (return bool _return_value)\n\
"},
  {NULL, NULL}};

static struct PyMethodDef _PetscOptionsModuleMethods[] = {{NULL, NULL}};

static void sidl_dealloc(struct PETSc_PetscOptions_PythonObject *self)
{
  if (self->weakRefList)
  {
    PyObject_ClearWeakRefs((PyObject *) self);
  }
  if ((self->refType != WEAK_REF) && (self->ior))
  {
    PetscObjectDestroy(self->ior);
  }
  self->ob_type->tp_free(self);
}

static int sidl_cmp(struct PETSc_PetscOptions_PythonObject *self, PyObject *arg)
{
  PetscObject ior1 = self->ior;
  PetscObject ior2 = (PetscObject) ((struct PETSc_PetscOptions_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (arg->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for sidl_cmp");
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  _return_value = PyObject_Compare(o1, o2);
  Py_DECREF(o1);
  Py_DECREF(o2);
  return _return_value;
}

static long sidl_hash(struct PETSc_PetscOptions_PythonObject *self)
{
  PetscObject ior = self->ior;
  PyObject *_hash_arg = NULL;
  long _return_value = 0;
  
  if (!ior)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_hash()");
    return _return_value;
  }
  _hash_arg = Py_BuildValue((char *) "i", ior);
  if (!_hash_arg)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not hash IOR pointer in sidl_hash()");
    return _return_value;
  }
  _return_value = PyObject_Hash(_hash_arg);
  Py_DECREF(_hash_arg);
  return _return_value;
}

static PyObject *sidl_str(struct PETSc_PetscOptions_PythonObject *self)
{
  const char *_str = NULL;
  PyObject *_return_value = NULL;
  
  _str = "PETSc.PetscOptions";
  _return_value = PyString_FromString(_str);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyObject *sidl_richcmp(struct PETSc_PetscOptions_PythonObject *self, PyObject *arg, int op)
{
  PetscObject ior1 = self->ior;
  PetscObject ior2 = (PetscObject) ((struct PETSc_PetscOptions_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  PyObject *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (!PyObject_HasAttrString(arg, (char *) "getClassName"))
  {
    switch(op) {
      case Py_EQ:
        _return_value = Py_False;
        Py_INCREF(_return_value);
        break;
      case Py_NE:
        _return_value = Py_True;
        Py_INCREF(_return_value);
        break;
      case Py_LT:
      case Py_LE:
      case Py_GT:
      case Py_GE:
        PyErr_SetString(PyExc_TypeError, (char *) "sidl_richcmp: Cannot order non-PETSc object");
    }
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  _return_value = PyObject_RichCompare(o1, o2, op);
  Py_DECREF(o1);
  Py_DECREF(o2);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyTypeObject SIDLPythonObjectType = {
  PyObject_HEAD_INIT(NULL)
  0,
  (char *) "PETSc.PetscOptions",
  sizeof(struct PETSc_PetscOptions_PythonObject),
  0,
  (destructor) sidl_dealloc,
  (printfunc) 0,
  (getattrfunc) 0,
  (setattrfunc) 0,
  (cmpfunc) sidl_cmp,
  (reprfunc) 0,
  (PyNumberMethods *) 0,
  (PySequenceMethods *) 0,
  (PyMappingMethods *) 0,
  (hashfunc) sidl_hash,
  (ternaryfunc) 0,
  (reprfunc) sidl_str,
  (getattrofunc) 0,
  (setattrofunc) 0,
  (PyBufferProcs *) 0,
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_WEAKREFS,
  (char *) "This is a Python wrapper for a SIDL object of type PETSc.PetscOptions",
  (traverseproc) 0,
  (inquiry) 0,
  (richcmpfunc) sidl_richcmp,
  (int) &((struct PETSc_PetscOptions_PythonObject *) 0)->weakRefList,
  (getiterfunc) 0,
  (iternextfunc) 0,
  _PetscOptionsObjectMethods,
  (struct PyMemberDef *) 0,
  (struct PyGetSetDef *) 0,
  0,
  0,
  (descrgetfunc) 0,
  (descrsetfunc) 0,
  0,
  (initproc) 0,
  (allocfunc) 0,
  (newfunc) PETSc_PetscOptions_PetscOptions,
  (freefunc) 0,
  (inquiry) 0};

void initPetscOptions(void)
{
  PyObject *module = NULL;
  
  /* Setup the python type */
  if (PyType_Ready(&SIDLPythonObjectType) < 0)
  {
    return;
  }
  /* Initialize the module */
  module = Py_InitModule3((char *) "PetscOptions", _PetscOptionsModuleMethods, (char *) "");
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Could not initialize module PETSc.PetscOptions");
    return;
  }
  /* Add the type to the module and increment the ref count */
  Py_INCREF(&SIDLPythonObjectType);
  PyModule_AddObject(module, (char *) "PetscOptions", (PyObject *) &SIDLPythonObjectType);
  /* Import Numeric */
  import_array();
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Cannot import Numeric Python array module");
    return;
  }
}
