#include <Python.h>

#include <Numeric/arrayobject.h>

#ifndef __PETSCSNES_H
#include <petscsnes.h>
#endif

enum ReferenceType {STOLEN_REF, WEAK_REF, NEW_REF};

struct PETSc_SNES_PythonObject {
  PyObject_HEAD;
  SNES ior;
  enum ReferenceType refType;
  PyObject *weakRefList;
};

/* Need to forward declare the type structure */
static PyTypeObject SIDLPythonObjectType;

static PyObject *sidl_new(PyTypeObject *type, PetscObject ior, enum ReferenceType refType)
{
  PyObject *return_value = NULL;
  
  if (ior)
  {
    struct PETSc_SNES_PythonObject *self = (struct PETSc_SNES_PythonObject *) type->tp_alloc(type, 0);
    
    self->weakRefList = NULL;
    if (self)
    {
      self->ior = (SNES) ior;
      self->refType = refType;
      if (refType == NEW_REF)
      {
        PetscObjectReference(ior);
      }
      return_value = (PyObject *) self;
    }
    else
    {
      PyErr_SetString(PyExc_MemoryError, (char *) "Unable to allocate interface object");
    }
  }
  else
  {
    return_value = Py_None;
    Py_INCREF(return_value);
  }
  return return_value;
}

static PyObject *opaque__wrap_ior(void *ptr)
{
  if (ptr)
  {
    return PyCObject_FromVoidPtr(ptr, NULL);
  }
  else
  {
    Py_INCREF(Py_None);
    return Py_None;
  }
}

static PyObject *PETSc_KSP__wrap_ior(KSP ior)
{
  PyObject *module = PyImport_ImportModule((char *) "PETSc.KSP");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "KSP");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module PETSc.KSP");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type PETSc.KSP");
    return 0;
  }
  return sidl_new(type, (PetscObject) ior, STOLEN_REF);
}

static PyObject *PETSc_Mat__wrap_ior(Mat ior)
{
  PyObject *module = PyImport_ImportModule((char *) "PETSc.Mat");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Mat");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module PETSc.Mat");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type PETSc.Mat");
    return 0;
  }
  return sidl_new(type, (PetscObject) ior, STOLEN_REF);
}

static PyObject *PETSc_SNES__wrap_ior(SNES ior)
{
  PyObject *module = PyImport_ImportModule((char *) "PETSc.SNES");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "SNES");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module PETSc.SNES");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type PETSc.SNES");
    return 0;
  }
  return sidl_new(type, (PetscObject) ior, STOLEN_REF);
}

static PyObject *PETSc_Vec__wrap_ior(Vec ior)
{
  PyObject *module = PyImport_ImportModule((char *) "PETSc.Vec");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Vec");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module PETSc.Vec");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type PETSc.Vec");
    return 0;
  }
  return sidl_new(type, (PetscObject) ior, STOLEN_REF);
}

static long PETSc_Mat__extract_ior(struct PETSc_SNES_PythonObject *obj, Mat *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (Mat) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "Mat"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.Mat");
    return 0;
  }
  return 1;
}

static long PETSc_SNES__extract_ior(struct PETSc_SNES_PythonObject *obj, SNES *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (SNES) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "SNES"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.SNES");
    return 0;
  }
  return 1;
}

static long PETSc_PetscObject__extract_ior(struct PETSc_SNES_PythonObject *obj, PetscObject *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (PetscObject) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "PetscObject"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.PetscObject");
    return 0;
  }
  return 1;
}

static long PETSc_PetscViewer__extract_ior(struct PETSc_SNES_PythonObject *obj, PetscViewer *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (PetscViewer) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "PetscViewer"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.PetscViewer");
    return 0;
  }
  return 1;
}

static long PETSc_Vec__extract_ior(struct PETSc_SNES_PythonObject *obj, Vec *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (Vec) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "Vec"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.Vec");
    return 0;
  }
  return 1;
}

static long opaque__extract_ior(PyObject *obj, void **ptr)
{
  if (PyCObject_Check(obj))
  {
    *ptr = PyCObject_AsVoidPtr(obj);
    return 1;
  }
  /* Why in the hell is there no Python interface to CObject? */
  if (PyObject_HasAttrString(obj, "handle"))
  {
    *ptr = PyLong_AsVoidPtr(PyObject_GetAttrString(obj, "handle"));
    return 1;
  }
  if (obj == Py_None)
  {
    *ptr = 0;
    return 1;
  }
  return 0;
}

#undef __FUNCT__

#define __FUNCT__ "PythonSNESFunction"

static int32_t PythonSNESFunction(SNES snes, Vec x, Vec f, PyObject *ctx)
{
  PyObject *_method = (PyObject *) ctx;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  PyObject *returnTuple = NULL;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  if (_method == Py_None)
  {
    return 0;
  }
  Py_INCREF(_method);
  if (snes)
  {
    PetscObjectReference(((PetscObject) snes));
  }
  if (x)
  {
    PetscObjectReference(((PetscObject) x));
  }
  if (f)
  {
    PetscObjectReference(((PetscObject) f));
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&O&O&)", (void *) PETSc_SNES__wrap_ior, snes, (void *) PETSc_Vec__wrap_ior, x, (void *) PETSc_Vec__wrap_ior, f);
  if (!_args)
  {
    SETERRQ(PETSC_ERR_LIB, "Unable to parse arguments for PETSc.SNES.PythonSNESFunction");
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (!_pyReturn)
  {
    PyErr_Print();
    return _return_value;
  }
  returnTuple = PyTuple_New(1);
  PyTuple_SetItem(returnTuple, 0, _pyReturn);
  /* Parse arguments */
  _okay = PyArg_ParseTuple(returnTuple, (char *) "l", &_local__return_value);
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_okay)
  {
    SETERRQ(PETSC_ERR_LIB, "Could not parse return value for PETSc.SNES.PythonSNESFunction");
  }
  /* Convert long to int32_t */
  _return_value = _local__return_value;
  /* Cleanup local return value */
  Py_DECREF(returnTuple);
  /* Cleanup arguments */
  Py_DECREF(_args);
  return _return_value;
}

#undef __FUNCT__

#define __FUNCT__ "PythonSNESJacobian"

static int32_t PythonSNESJacobian(SNES snes, Vec x, Mat J, Mat M, MatStructure flag, PyObject *ctx)
{
  PyObject *_method = (PyObject *) ctx;
  long _local_flag = 0;
  PyObject *_args = NULL;
  long _okay = 0;
  PyObject *_pyReturn = NULL;
  PyObject *returnTuple = NULL;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  if (_method == Py_None)
  {
    return 0;
  }
  Py_INCREF(_method);
  J = *((Mat *) J);
  M = *((Mat *) M);
  if (snes)
  {
    PetscObjectReference(((PetscObject) snes));
  }
  if (x)
  {
    PetscObjectReference(((PetscObject) x));
  }
  if (J)
  {
    PetscObjectReference(((PetscObject) J));
  }
  if (M)
  {
    PetscObjectReference(((PetscObject) M));
  }
  /* Convert MatStructure to long */
  _local_flag = flag;
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&O&O&O&l)", (void *) PETSc_SNES__wrap_ior, snes, (void *) PETSc_Vec__wrap_ior, x, (void *) PETSc_Mat__wrap_ior, J, (void *) PETSc_Mat__wrap_ior, M, _local_flag);
  if (!_args)
  {
    SETERRQ(PETSC_ERR_LIB, "Unable to parse arguments for PETSc.SNES.PythonSNESJacobian");
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (!_pyReturn)
  {
    PyErr_Print();
    return _return_value;
  }
  returnTuple = PyTuple_New(1);
  PyTuple_SetItem(returnTuple, 0, _pyReturn);
  /* Parse arguments */
  _okay = PyArg_ParseTuple(returnTuple, (char *) "l", &_local__return_value);
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_okay)
  {
    SETERRQ(PETSC_ERR_LIB, "Could not parse return value for PETSc.SNES.PythonSNESJacobian");
  }
  /* Convert long to int32_t */
  _return_value = _local__return_value;
  /* Cleanup local return value */
  Py_DECREF(returnTuple);
  /* Cleanup arguments */
  Py_DECREF(_args);
  return _return_value;
}

static PyObject *PETSc_PetscObject_getType(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getType");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getType");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESGetType(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setType(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "typeName",
    NULL};
  long _okay = 0;
  const char *typeName = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setType");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setType");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &typeName);
  /* Copying the Python string to IOR controlled memory */
  typeName = typeName ? strcpy(malloc(strlen(typeName) + 1), typeName) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESSetType(_self_ior, typeName);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) typeName);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_getOptionsPrefix(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getOptionsPrefix");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getOptionsPrefix");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESGetOptionsPrefix(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setOptionsPrefix(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "prefix",
    NULL};
  long _okay = 0;
  const char *prefix = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setOptionsPrefix");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setOptionsPrefix");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &prefix);
  /* Copying the Python string to IOR controlled memory */
  prefix = prefix ? strcpy(malloc(strlen(prefix) + 1), prefix) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESSetOptionsPrefix(_self_ior, prefix);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) prefix);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_appendOptionsPrefix(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "prefix",
    NULL};
  long _okay = 0;
  const char *prefix = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for appendOptionsPrefix");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in appendOptionsPrefix");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &prefix);
  /* Copying the Python string to IOR controlled memory */
  prefix = prefix ? strcpy(malloc(strlen(prefix) + 1), prefix) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESAppendOptionsPrefix(_self_ior, prefix);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) prefix);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setFromOptions(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setFromOptions");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setFromOptions");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESSetFromOptions(_self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setUp(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setUp");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setUp");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESSetUp(_self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_view(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "viewer",
    NULL};
  long _okay = 0;
  PetscViewer viewer = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for view");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in view");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_PetscViewer__extract_ior, &viewer);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESView(_self_ior, viewer);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_getObject(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  void *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getObject");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getObject");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) opaque__wrap_ior, _return_value);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setObject(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "object",
    NULL};
  long _okay = 0;
  void *object = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setObject");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setObject");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) opaque__extract_ior, &object);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  if ((((struct PETSc_SNES_PythonObject *) _self)->refType != WEAK_REF) && (((struct PETSc_SNES_PythonObject *) _self)->ior))
  {
    SNESDestroy(((struct PETSc_SNES_PythonObject *) _self)->ior);
  }
  ((struct PETSc_SNES_PythonObject *) _self)->ior = object;
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getReferenceCount(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getReferenceCount");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getReferenceCount");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectGetReference((PetscObject) _self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_addReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for addReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in addReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectReference((PetscObject) _self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_deleteReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for deleteReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in deleteReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectDereference((PetscObject) _self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getClassName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  const char *_return_value = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getClassName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getClassName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = ((PetscObject) _self_ior)->class_name;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectGetName((PetscObject) _self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_setName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "name",
    NULL};
  long _okay = 0;
  const char *name = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &name);
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectSetName((PetscObject) _self_ior, name);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) name);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_isInstanceOf(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "typeName",
    NULL};
  long _okay = 0;
  const char *typeName = NULL;
  PetscTruth _return_value = 0;
  PetscTruth _local__return_value = 0;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isInstanceOf");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in isInstanceOf");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &typeName);
  /* Copying the Python string to IOR controlled memory */
  typeName = typeName ? strcpy(malloc(strlen(typeName) + 1), typeName) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = !strcmp(((PetscObject) _self_ior)->class_name, typeName);
  /* Convert PetscTruth to PetscTruth */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local__return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) typeName);
  return _pyReturn;
}

static PyObject *PETSc_SNES_getKSP(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  KSP _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getKSP");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getKSP");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESGetKSP(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) PETSc_KSP__wrap_ior, _return_value);
  if (_return_value)
  {
    PetscObjectReference(((PetscObject) _return_value));
  }
  return _pyReturn;
}

static PyObject *PETSc_SNES_getRhs(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  Vec _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getRhs");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getRhs");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESGetRhs(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) PETSc_Vec__wrap_ior, _return_value);
  if (_return_value)
  {
    PetscObjectReference(((PetscObject) _return_value));
  }
  return _pyReturn;
}

static PyObject *PETSc_SNES_setRhs(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "b",
    NULL};
  long _okay = 0;
  Vec b = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setRhs");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setRhs");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_Vec__extract_ior, &b);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESSetRhs(_self_ior, b);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_SNES_getFunction(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  Vec f = 0;
  PyObject *func = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getFunction");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getFunction");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESGetFunction(_self_ior, &f, NULL, (void **) &func);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "(O&O)", (void *) PETSc_Vec__wrap_ior, f, func);
  if (f)
  {
    PetscObjectReference(((PetscObject) f));
  }
  return _pyReturn;
}

static PyObject *PETSc_SNES_setFunction(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "f",
    (char *) "func",
    NULL};
  long _okay = 0;
  Vec f = 0;
  PyObject *func = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setFunction");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setFunction");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O", _kwlist, (void *) PETSc_Vec__extract_ior, &f, &func);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESSetFunction(_self_ior, f, PythonSNESFunction, func);
  /* PETSc must hold a reference to the method object (these leak right now) */
  Py_INCREF(func);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_SNES_getJacobian(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  Mat J = 0;
  Mat M = 0;
  PyObject *func = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getJacobian");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getJacobian");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESGetJacobian(_self_ior, &J, &M, NULL, (void **) &func);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "(O&O&O)", (void *) PETSc_Mat__wrap_ior, J, (void *) PETSc_Mat__wrap_ior, M, func);
  if (J)
  {
    PetscObjectReference(((PetscObject) J));
  }
  if (M)
  {
    PetscObjectReference(((PetscObject) M));
  }
  return _pyReturn;
}

static PyObject *PETSc_SNES_setJacobian(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "J",
    (char *) "M",
    (char *) "jacobian",
    NULL};
  long _okay = 0;
  Mat J = 0;
  Mat M = 0;
  PyObject *jacobian = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setJacobian");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setJacobian");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&O", _kwlist, (void *) PETSc_Mat__extract_ior, &J, (void *) PETSc_Mat__extract_ior, &M, &jacobian);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESSetJacobian(_self_ior, J, M, PythonSNESJacobian, jacobian);
  /* PETSc must hold a reference to the method object (these leak right now) */
  Py_INCREF(jacobian);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_SNES_computeFunction(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "x",
    (char *) "f",
    NULL};
  long _okay = 0;
  Vec x = 0;
  Vec f = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for computeFunction");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in computeFunction");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&", _kwlist, (void *) PETSc_Vec__extract_ior, &x, (void *) PETSc_Vec__extract_ior, &f);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESComputeFunction(_self_ior, x, f);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_SNES_computeJacobian(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "x",
    NULL};
  long _okay = 0;
  Vec x = 0;
  int ierr = 0;
  Mat J = 0;
  Mat M = 0;
  MatStructure flag = 0;
  long _local_flag = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for computeJacobian");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in computeJacobian");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_Vec__extract_ior, &x);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESComputeJacobian(_self_ior, x, &J, &M, &flag);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert MatStructure to long */
  _local_flag = flag;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "(O&O&l)", (void *) PETSc_Mat__wrap_ior, J, (void *) PETSc_Mat__wrap_ior, M, _local_flag);
  if (J)
  {
    PetscObjectReference(((PetscObject) J));
  }
  if (M)
  {
    PetscObjectReference(((PetscObject) M));
  }
  return _pyReturn;
}

static PyObject *PETSc_SNES_getTolerances(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  double abstol = 0.0;
  double rtol = 0.0;
  double stol = 0.0;
  int32_t maxit = 0;
  long _local_maxit = 0;
  int32_t maxf = 0;
  long _local_maxf = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getTolerances");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getTolerances");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESGetTolerances(_self_ior, &abstol, &rtol, &stol, &maxit, &maxf);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local_maxit = maxit;
  /* Convert int32_t to long */
  _local_maxf = maxf;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "(dddll)", abstol, rtol, stol, _local_maxit, _local_maxf);
  return _pyReturn;
}

static PyObject *PETSc_SNES_setTolerances(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "abstol",
    (char *) "rtol",
    (char *) "stol",
    (char *) "maxit",
    (char *) "maxf",
    NULL};
  long _okay = 0;
  double abstol = 0.0;
  double rtol = 0.0;
  double stol = 0.0;
  int32_t maxit = 0;
  long _local_maxit = 0;
  int32_t maxf = 0;
  long _local_maxf = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setTolerances");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setTolerances");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "dddll", _kwlist, &abstol, &rtol, &stol, &_local_maxit, &_local_maxf);
  /* Convert long to int32_t */
  maxit = _local_maxit;
  /* Convert long to int32_t */
  maxf = _local_maxf;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESSetTolerances(_self_ior, abstol, rtol, stol, maxit, maxf);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_SNES_solve(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "b",
    (char *) "x",
    NULL};
  long _okay = 0;
  Vec b = 0;
  Vec x = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for solve");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in solve");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&", _kwlist, (void *) PETSc_Vec__extract_ior, &b, (void *) PETSc_Vec__extract_ior, &x);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESSolve(_self_ior, b, x);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_SNES_getSolution(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  Vec _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getSolution");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getSolution");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESGetSolution(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) PETSc_Vec__wrap_ior, _return_value);
  if (_return_value)
  {
    PetscObjectReference(((PetscObject) _return_value));
  }
  return _pyReturn;
}

static PyObject *PETSc_SNES_getConvergedReason(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  SNESConvergedReason _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getConvergedReason");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getConvergedReason");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESGetConvergedReason(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert SNESConvergedReason to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *PETSc_SNES_getIterationNumber(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getIterationNumber");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getIterationNumber");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESGetIterationNumber(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *PETSc_SNES_getNumberLinearIterations(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getNumberLinearIterations");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getNumberLinearIterations");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESGetNumberLinearIterations(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *PETSc_SNES_getFunctionNorm(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  double _return_value = 0.0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getFunctionNorm");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getFunctionNorm");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = SNESGetFunctionNorm(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "d", _return_value);
  return _pyReturn;
}

static PyObject *PETSc_SNES_SNES(PyObject *_type, PyObject *_args, PyObject *_kwdict)
{
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  long _okay = 0;
  SNES obj = 0;
  PetscTruth created = 0;
  SNES _return_value = 0;
  PyObject *_pyReturn = NULL;
  
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "|O&", _kwlist, (void *) PETSc_SNES__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  if (obj)
  {
    created = 0;
  }
  else
  {
    created = 1;
    SNESCreate(PETSC_COMM_WORLD, &obj);
  }
  _return_value = obj;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) PETSc_SNES__wrap_ior, _return_value);
  if (_return_value)
  {
    PetscObjectReference(((PetscObject) _return_value));
  }
  if (created)
  {
    PetscObjectDereference((PetscObject) obj);
  }
  return _pyReturn;
}

static PyObject *isSame(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  /* Retrieve the IOR pointer */
  SNES _self_ior = ((struct PETSc_SNES_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  long _okay = 0;
  PetscObject obj = 0;
  PetscTruth _return_value = 0;
  PetscTruth _local__return_value = 0;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isSame");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in isSame");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_PetscObject__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = (_self_ior == (SNES) obj);
  /* Convert PetscTruth to PetscTruth */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local__return_value);
  return _pyReturn;
}

static struct PyMethodDef _SNESObjectMethods[] = {
  {(char *) "getType", (PyCFunction) PETSc_PetscObject_getType, (METH_VARARGS | METH_KEYWORDS), (char *) "getType()\n\
RETURNS (return string _return_value)\n\
 Get the implementation type"},
  {(char *) "setType", (PyCFunction) PETSc_PetscObject_setType, (METH_VARARGS | METH_KEYWORDS), (char *) "setType(in string typeName)\n\
RETURNS None\n\
\n\
 Set the implementation type"},
  {(char *) "getOptionsPrefix", (PyCFunction) PETSc_PetscObject_getOptionsPrefix, (METH_VARARGS | METH_KEYWORDS), (char *) "getOptionsPrefix()\n\
RETURNS (return string _return_value)\n\
 Get the prefix appended to options directed to this object"},
  {(char *) "setOptionsPrefix", (PyCFunction) PETSc_PetscObject_setOptionsPrefix, (METH_VARARGS | METH_KEYWORDS), (char *) "setOptionsPrefix(in string prefix)\n\
RETURNS None\n\
\n\
 Set the prefix appended to options directed to this object"},
  {(char *) "appendOptionsPrefix", (PyCFunction) PETSc_PetscObject_appendOptionsPrefix, (METH_VARARGS | METH_KEYWORDS), (char *) "appendOptionsPrefix(in string prefix)\n\
RETURNS None\n\
\n\
 Enlarge the prefix appended to options directed to this object"},
  {(char *) "setFromOptions", (PyCFunction) PETSc_PetscObject_setFromOptions, (METH_VARARGS | METH_KEYWORDS), (char *) "setFromOptions()\n\
RETURNS None\n\
\n\
 Configure the object using the PETSc options database"},
  {(char *) "setUp", (PyCFunction) PETSc_PetscObject_setUp, (METH_VARARGS | METH_KEYWORDS), (char *) "setUp()\n\
RETURNS None\n\
\n\
 Initiate the object setup phase"},
  {(char *) "view", (PyCFunction) PETSc_PetscObject_view, (METH_VARARGS | METH_KEYWORDS), (char *) "view(in PETSc.PetscViewer viewer)\n\
RETURNS None\n\
\n\
 Output a representation of the object using "},
  {(char *) "getObject", (PyCFunction) PETSc_PetscObject_getObject, (METH_VARARGS | METH_KEYWORDS), (char *) "getObject()\n\
RETURNS (return opaque _return_value)\n\
 Get the PETSc object pointer"},
  {(char *) "setObject", (PyCFunction) PETSc_PetscObject_setObject, (METH_VARARGS | METH_KEYWORDS), (char *) "setObject(in opaque object)\n\
RETURNS None\n\
\n\
 Set the PETSc object pointer"},
  {(char *) "getReferenceCount", (PyCFunction) ASE_BaseInterface_getReferenceCount, (METH_VARARGS | METH_KEYWORDS), (char *) "getReferenceCount()\n\
RETURNS (return int _return_value)\n\
 Retrieve the intrinsic reference count of the underlying object"},
  {(char *) "addReference", (PyCFunction) ASE_BaseInterface_addReference, (METH_VARARGS | METH_KEYWORDS), (char *) "addReference()\n\
RETURNS None\n\
\n\
 Add one to the intrinsic reference count of the underlying object - Objects continue to exist as long as the reference count is positive - Clients should call this method whenever they create another ongoing reference to an object or interface"},
  {(char *) "deleteReference", (PyCFunction) ASE_BaseInterface_deleteReference, (METH_VARARGS | METH_KEYWORDS), (char *) "deleteReference()\n\
RETURNS None\n\
\n\
 Decrease by one the intrinsic reference count of the underlying object - Delete the object if the reference count is non-positive - Clients should call this method whenever they remove a reference to an object or interface"},
  {(char *) "getClassName", (PyCFunction) ASE_BaseInterface_getClassName, (METH_VARARGS | METH_KEYWORDS), (char *) "getClassName()\n\
RETURNS (return string _return_value)\n\
 Return the type name of the underlying class object"},
  {(char *) "getName", (PyCFunction) ASE_BaseInterface_getName, (METH_VARARGS | METH_KEYWORDS), (char *) "getName()\n\
RETURNS (return string _return_value)\n\
 Retrieve the object name"},
  {(char *) "setName", (PyCFunction) ASE_BaseInterface_setName, (METH_VARARGS | METH_KEYWORDS), (char *) "setName(in string name)\n\
RETURNS None\n\
\n\
 Set the object name"},
  {(char *) "isInstanceOf", (PyCFunction) ASE_BaseInterface_isInstanceOf, (METH_VARARGS | METH_KEYWORDS), (char *) "isInstanceOf(in string typeName)\n\
RETURNS (return bool _return_value)\n\
 Return <code>true</code> if this object is an instance of the specified type - This routine will return <code>true</code> if and only if a cast to the specified type would succeed"},
  {(char *) "getKSP", (PyCFunction) PETSc_SNES_getKSP, (METH_VARARGS | METH_KEYWORDS), (char *) "getKSP()\n\
RETURNS (return PETSc.KSP _return_value)\n\
 Retrieve the linear solver"},
  {(char *) "getRhs", (PyCFunction) PETSc_SNES_getRhs, (METH_VARARGS | METH_KEYWORDS), (char *) "getRhs()\n\
RETURNS (return PETSc.Vec _return_value)\n\
 Get the constant part"},
  {(char *) "setRhs", (PyCFunction) PETSc_SNES_setRhs, (METH_VARARGS | METH_KEYWORDS), (char *) "setRhs(in PETSc.Vec b)\n\
RETURNS None\n\
\n\
 Set the constant part"},
  {(char *) "getFunction", (PyCFunction) PETSc_SNES_getFunction, (METH_VARARGS | METH_KEYWORDS), (char *) "getFunction(out PETSc.Vec f, out function func)\n\
RETURNS (out PETSc.Vec f, out function func)\n\
 Get the function which calculates the residual F"},
  {(char *) "setFunction", (PyCFunction) PETSc_SNES_setFunction, (METH_VARARGS | METH_KEYWORDS), (char *) "setFunction(in PETSc.Vec f, in function func)\n\
RETURNS None\n\
\n\
 Set the function which calculates the residual F"},
  {(char *) "getJacobian", (PyCFunction) PETSc_SNES_getJacobian, (METH_VARARGS | METH_KEYWORDS), (char *) "getJacobian(out PETSc.Mat J, out PETSc.Mat M, out function func)\n\
RETURNS (out PETSc.Mat J, out PETSc.Mat M, out function func)\n\
 Get the function which calculates the Jacobian J"},
  {(char *) "setJacobian", (PyCFunction) PETSc_SNES_setJacobian, (METH_VARARGS | METH_KEYWORDS), (char *) "setJacobian(in PETSc.Mat J, in PETSc.Mat M, in function jacobian)\n\
RETURNS None\n\
\n\
 Set the function which calculates the jacobian J"},
  {(char *) "computeFunction", (PyCFunction) PETSc_SNES_computeFunction, (METH_VARARGS | METH_KEYWORDS), (char *) "computeFunction(in PETSc.Vec x, in PETSc.Vec f)\n\
RETURNS None\n\
\n\
 Compute the residual F(x)"},
  {(char *) "computeJacobian", (PyCFunction) PETSc_SNES_computeJacobian, (METH_VARARGS | METH_KEYWORDS), (char *) "computeJacobian(in PETSc.Vec x, out PETSc.Mat J, out PETSc.Mat M, out PETSc.MatStructure flag)\n\
RETURNS (out PETSc.Mat J, out PETSc.Mat M, out PETSc.MatStructure flag)\n\
 Compute the jacobian J(x)"},
  {(char *) "getTolerances", (PyCFunction) PETSc_SNES_getTolerances, (METH_VARARGS | METH_KEYWORDS), (char *) "getTolerances(out double abstol, out double rtol, out double stol, out int maxit, out int maxf)\n\
RETURNS (out double abstol, out double rtol, out double stol, out int maxit, out int maxf)\n\
 Get the solver tolerances:   absolute residual tolerance   relative residual tolerance   absolute solution update tolerance   maximum iteration count   maximum function evaluation count"},
  {(char *) "setTolerances", (PyCFunction) PETSc_SNES_setTolerances, (METH_VARARGS | METH_KEYWORDS), (char *) "setTolerances(in double abstol, in double rtol, in double stol, in int maxit, in int maxf)\n\
RETURNS None\n\
\n\
 Set the solver tolerances:   absolute residual tolerance   relative residual tolerance   absolute solution update tolerance   maximum iteration count   maximum function evaluation count"},
  {(char *) "solve", (PyCFunction) PETSc_SNES_solve, (METH_VARARGS | METH_KEYWORDS), (char *) "solve(in PETSc.Vec b, in PETSc.Vec x)\n\
RETURNS None\n\
\n\
 Solve the system of nonlinear algebraic equations"},
  {(char *) "getSolution", (PyCFunction) PETSc_SNES_getSolution, (METH_VARARGS | METH_KEYWORDS), (char *) "getSolution()\n\
RETURNS (return PETSc.Vec _return_value)\n\
 Get the solution vector"},
  {(char *) "getConvergedReason", (PyCFunction) PETSc_SNES_getConvergedReason, (METH_VARARGS | METH_KEYWORDS), (char *) "getConvergedReason()\n\
RETURNS (return PETSc.SNESConvergedReason _return_value)\n\
 Get the reason for termination of the solver"},
  {(char *) "getIterationNumber", (PyCFunction) PETSc_SNES_getIterationNumber, (METH_VARARGS | METH_KEYWORDS), (char *) "getIterationNumber()\n\
RETURNS (return int _return_value)\n\
 Get the number of solver iterates"},
  {(char *) "getNumberLinearIterations", (PyCFunction) PETSc_SNES_getNumberLinearIterations, (METH_VARARGS | METH_KEYWORDS), (char *) "getNumberLinearIterations()\n\
RETURNS (return int _return_value)\n\
 Get the number of linear solver iterates"},
  {(char *) "getFunctionNorm", (PyCFunction) PETSc_SNES_getFunctionNorm, (METH_VARARGS | METH_KEYWORDS), (char *) "getFunctionNorm()\n\
RETURNS (return double _return_value)\n\
 Get the residual norm"},
  {(char *) "isSame", (PyCFunction) isSame, (METH_VARARGS | METH_KEYWORDS), (char *) "isSame(in PETSc.PetscObject obj)\n\
RETURNS (return bool _return_value)\n\
"},
  {NULL, NULL}};

static struct PyMethodDef _SNESModuleMethods[] = {{NULL, NULL}};

static void sidl_dealloc(struct PETSc_SNES_PythonObject *self)
{
  if (self->weakRefList)
  {
    PyObject_ClearWeakRefs((PyObject *) self);
  }
  if ((self->refType != WEAK_REF) && (self->ior))
  {
    SNESDestroy(self->ior);
  }
  self->ob_type->tp_free(self);
}

static int sidl_cmp(struct PETSc_SNES_PythonObject *self, PyObject *arg)
{
  SNES ior1 = self->ior;
  PetscObject ior2 = (PetscObject) ((struct PETSc_SNES_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (arg->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for sidl_cmp");
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  _return_value = PyObject_Compare(o1, o2);
  Py_DECREF(o1);
  Py_DECREF(o2);
  return _return_value;
}

static long sidl_hash(struct PETSc_SNES_PythonObject *self)
{
  SNES ior = self->ior;
  PyObject *_hash_arg = NULL;
  long _return_value = 0;
  
  if (!ior)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_hash()");
    return _return_value;
  }
  _hash_arg = Py_BuildValue((char *) "i", ior);
  if (!_hash_arg)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not hash IOR pointer in sidl_hash()");
    return _return_value;
  }
  _return_value = PyObject_Hash(_hash_arg);
  Py_DECREF(_hash_arg);
  return _return_value;
}

static PyObject *sidl_str(struct PETSc_SNES_PythonObject *self)
{
  const char *_str = NULL;
  PyObject *_return_value = NULL;
  SNES ior = self->ior;
  
  SNESGetType(ior, (SNESType *) &_str);
  _return_value = PyString_FromString(_str);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyObject *sidl_richcmp(struct PETSc_SNES_PythonObject *self, PyObject *arg, int op)
{
  SNES ior1 = self->ior;
  PetscObject ior2 = (PetscObject) ((struct PETSc_SNES_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  PyObject *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (!PyObject_HasAttrString(arg, (char *) "getClassName"))
  {
    switch(op) {
      case Py_EQ:
        _return_value = Py_False;
        Py_INCREF(_return_value);
        break;
      case Py_NE:
        _return_value = Py_True;
        Py_INCREF(_return_value);
        break;
      case Py_LT:
      case Py_LE:
      case Py_GT:
      case Py_GE:
        PyErr_SetString(PyExc_TypeError, (char *) "sidl_richcmp: Cannot order non-PETSc object");
    }
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  _return_value = PyObject_RichCompare(o1, o2, op);
  Py_DECREF(o1);
  Py_DECREF(o2);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyTypeObject SIDLPythonObjectType = {
  PyObject_HEAD_INIT(NULL)
  0,
  (char *) "PETSc.SNES",
  sizeof(struct PETSc_SNES_PythonObject),
  0,
  (destructor) sidl_dealloc,
  (printfunc) 0,
  (getattrfunc) 0,
  (setattrfunc) 0,
  (cmpfunc) sidl_cmp,
  (reprfunc) 0,
  (PyNumberMethods *) 0,
  (PySequenceMethods *) 0,
  (PyMappingMethods *) 0,
  (hashfunc) sidl_hash,
  (ternaryfunc) 0,
  (reprfunc) sidl_str,
  (getattrofunc) 0,
  (setattrofunc) 0,
  (PyBufferProcs *) 0,
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_WEAKREFS,
  (char *) "This is a Python wrapper for a SIDL object of type PETSc.SNES",
  (traverseproc) 0,
  (inquiry) 0,
  (richcmpfunc) sidl_richcmp,
  (int) &((struct PETSc_SNES_PythonObject *) 0)->weakRefList,
  (getiterfunc) 0,
  (iternextfunc) 0,
  _SNESObjectMethods,
  (struct PyMemberDef *) 0,
  (struct PyGetSetDef *) 0,
  0,
  0,
  (descrgetfunc) 0,
  (descrsetfunc) 0,
  0,
  (initproc) 0,
  (allocfunc) 0,
  (newfunc) PETSc_SNES_SNES,
  (freefunc) 0,
  (inquiry) 0};

void initSNES(void)
{
  PyObject *module = NULL;
  
  /* Setup the python type */
  if (PyType_Ready(&SIDLPythonObjectType) < 0)
  {
    return;
  }
  /* Initialize the module */
  module = Py_InitModule3((char *) "SNES", _SNESModuleMethods, (char *) "");
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Could not initialize module PETSc.SNES");
    return;
  }
  /* Add the type to the module and increment the ref count */
  Py_INCREF(&SIDLPythonObjectType);
  PyModule_AddObject(module, (char *) "SNES", (PyObject *) &SIDLPythonObjectType);
  /* Import Numeric */
  import_array();
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Cannot import Numeric Python array module");
    return;
  }
}
