#include <Python.h>

#include <Numeric/arrayobject.h>

#ifndef __PETSCVEC_H
#include <petscvec.h>
#endif

enum ReferenceType {STOLEN_REF, WEAK_REF, NEW_REF};

struct PETSc_Vec_PythonObject {
  PyObject_HEAD;
  Vec ior;
  enum ReferenceType refType;
  PyObject *weakRefList;
};

/* Need to forward declare the type structure */
static PyTypeObject SIDLPythonObjectType;

static PyObject *sidl_new(PyTypeObject *type, PetscObject ior, enum ReferenceType refType)
{
  PyObject *return_value = NULL;
  
  if (ior)
  {
    struct PETSc_Vec_PythonObject *self = (struct PETSc_Vec_PythonObject *) type->tp_alloc(type, 0);
    
    self->weakRefList = NULL;
    if (self)
    {
      self->ior = (Vec) ior;
      self->refType = refType;
      if (refType == NEW_REF)
      {
        PetscObjectReference(ior);
      }
      return_value = (PyObject *) self;
    }
    else
    {
      PyErr_SetString(PyExc_MemoryError, (char *) "Unable to allocate interface object");
    }
  }
  else
  {
    return_value = Py_None;
    Py_INCREF(return_value);
  }
  return return_value;
}

static PyObject *opaque__wrap_ior(void *ptr)
{
  if (ptr)
  {
    return PyCObject_FromVoidPtr(ptr, NULL);
  }
  else
  {
    Py_INCREF(Py_None);
    return Py_None;
  }
}

static PyObject *PETSc_Vec__wrap_ior(Vec ior)
{
  PyObject *module = PyImport_ImportModule((char *) "PETSc.Vec");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Vec");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module PETSc.Vec");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type PETSc.Vec");
    return 0;
  }
  return sidl_new(type, (PetscObject) ior, STOLEN_REF);
}

static PyObject *PETSc_PetscMap__wrap_ior(PetscMap ior)
{
  PyObject *module = PyImport_ImportModule((char *) "PETSc.PetscMap");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "PetscMap");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module PETSc.PetscMap");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type PETSc.PetscMap");
    return 0;
  }
  return sidl_new(type, (PetscObject) ior, STOLEN_REF);
}

static long PETSc_PetscViewer__extract_ior(struct PETSc_Vec_PythonObject *obj, PetscViewer *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (PetscViewer) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "PetscViewer"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.PetscViewer");
    return 0;
  }
  return 1;
}

static long opaque__extract_ior(PyObject *obj, void **ptr)
{
  if (PyCObject_Check(obj))
  {
    *ptr = PyCObject_AsVoidPtr(obj);
    return 1;
  }
  /* Why in the hell is there no Python interface to CObject? */
  if (PyObject_HasAttrString(obj, "handle"))
  {
    *ptr = PyLong_AsVoidPtr(PyObject_GetAttrString(obj, "handle"));
    return 1;
  }
  if (obj == Py_None)
  {
    *ptr = 0;
    return 1;
  }
  return 0;
}

static long PETSc_PetscObject__extract_ior(struct PETSc_Vec_PythonObject *obj, PetscObject *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (PetscObject) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "PetscObject"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.PetscObject");
    return 0;
  }
  return 1;
}

static long PETSc_Vec__extract_ior(struct PETSc_Vec_PythonObject *obj, Vec *ior)
{
  PetscObject objIor = (PetscObject) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (Vec) objIor;
  if (strcmp(((PetscObject) (*ior))->class_name, "Vec"))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type PETSc.Vec");
    return 0;
  }
  return 1;
}

struct petsc__array {
  void *data;
  int size;
};

static PyObject *double__wrap_array(struct petsc__array *array)
{
  PyArrayObject *_return_value = NULL;
  
  if (array->data)
  {
    int numElem[1] = {array->size};
    
    _return_value = (PyArrayObject *) PyArray_FromDimsAndData(1, numElem, PyArray_DOUBLE, array->data);
  }
  return (PyObject *) _return_value;
}

static long double__extract_array_borrow(PyObject *obj, struct petsc__array *array)
{
  /* This will create a new Numeric array from a Python sequence */
  PyArrayObject *pyArray = (PyArrayObject *) PyArray_FromObject(obj, PyArray_DOUBLE, 0, 0);
  long _return_value = 0;
  
  array->size = 0;
  array->data = NULL;
  if (pyArray)
  {
    if (pyArray->nd != 1)
    {
      return 0;
    }
    array->size = PyArray_Size((PyObject *) pyArray);
    array->data = pyArray->data;
    _return_value = 1;
  }
  return _return_value;
}

#ifndef __PETSCVEC_H
#include <petscvec.h>
#endif

static long PETSc_Vec__extract_array_copy(PyObject *obj, struct petsc__array *array)
{
  /* This will create a new Numeric array from a Python sequence */
  PyArrayObject *pyArray = (PyArrayObject *) PyArray_FromObject(obj, PyArray_OBJECT, 0, 0);
  long _return_value = 0;
  
  array->size = 0;
  array->data = NULL;
  if (pyArray)
  {
    Vec *pyData = (Vec *) pyArray->data;
    Vec *petscData = NULL;
    long i = 0;
    
    if (pyArray->nd != 1)
    {
      return 0;
    }
    array->size = PyArray_Size((PyObject *) pyArray);
    PetscMalloc(array->size * sizeof(Vec), &(array->data));
    petscData = (Vec *) array->data;
    for(i = 0; i < array->size; i++)
    {
      petscData[i] = pyData[i];
    }
    _return_value = 1;
  }
  return _return_value;
}

static long int__extract_array_borrow(PyObject *obj, struct petsc__array *array)
{
  /* This will create a new Numeric array from a Python sequence */
  PyArrayObject *pyArray = (PyArrayObject *) PyArray_FromObject(obj, PyArray_INT, 0, 0);
  long _return_value = 0;
  
  array->size = 0;
  array->data = NULL;
  if (pyArray)
  {
    if (pyArray->nd != 1)
    {
      return 0;
    }
    array->size = PyArray_Size((PyObject *) pyArray);
    array->data = pyArray->data;
    _return_value = 1;
  }
  return _return_value;
}

static PyObject *PETSc_PetscObject_getType(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getType");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getType");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecGetType(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setType(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "typeName",
    NULL};
  long _okay = 0;
  const char *typeName = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setType");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setType");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &typeName);
  /* Copying the Python string to IOR controlled memory */
  typeName = typeName ? strcpy(malloc(strlen(typeName) + 1), typeName) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecSetType(_self_ior, typeName);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) typeName);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_getOptionsPrefix(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getOptionsPrefix");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getOptionsPrefix");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecGetOptionsPrefix(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setOptionsPrefix(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "prefix",
    NULL};
  long _okay = 0;
  const char *prefix = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setOptionsPrefix");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setOptionsPrefix");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &prefix);
  /* Copying the Python string to IOR controlled memory */
  prefix = prefix ? strcpy(malloc(strlen(prefix) + 1), prefix) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecSetOptionsPrefix(_self_ior, prefix);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) prefix);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_appendOptionsPrefix(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "prefix",
    NULL};
  long _okay = 0;
  const char *prefix = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for appendOptionsPrefix");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in appendOptionsPrefix");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &prefix);
  /* Copying the Python string to IOR controlled memory */
  prefix = prefix ? strcpy(malloc(strlen(prefix) + 1), prefix) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecAppendOptionsPrefix(_self_ior, prefix);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) prefix);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setFromOptions(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setFromOptions");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setFromOptions");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecSetFromOptions(_self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setUp(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setUp");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setUp");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecSetUp(_self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_view(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "viewer",
    NULL};
  long _okay = 0;
  PetscViewer viewer = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for view");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in view");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_PetscViewer__extract_ior, &viewer);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecView(_self_ior, viewer);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_getObject(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  void *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getObject");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getObject");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) opaque__wrap_ior, _return_value);
  return _pyReturn;
}

static PyObject *PETSc_PetscObject_setObject(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "object",
    NULL};
  long _okay = 0;
  void *object = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setObject");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setObject");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) opaque__extract_ior, &object);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  if ((((struct PETSc_Vec_PythonObject *) _self)->refType != WEAK_REF) && (((struct PETSc_Vec_PythonObject *) _self)->ior))
  {
    VecDestroy(((struct PETSc_Vec_PythonObject *) _self)->ior);
  }
  ((struct PETSc_Vec_PythonObject *) _self)->ior = object;
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getReferenceCount(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getReferenceCount");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getReferenceCount");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectGetReference((PetscObject) _self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_addReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for addReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in addReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectReference((PetscObject) _self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_deleteReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for deleteReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in deleteReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectDereference((PetscObject) _self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getClassName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  const char *_return_value = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getClassName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getClassName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = ((PetscObject) _self_ior)->class_name;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectGetName((PetscObject) _self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_setName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "name",
    NULL};
  long _okay = 0;
  const char *name = NULL;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &name);
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = PetscObjectSetName((PetscObject) _self_ior, name);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) name);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_isInstanceOf(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "typeName",
    NULL};
  long _okay = 0;
  const char *typeName = NULL;
  PetscTruth _return_value = 0;
  PetscTruth _local__return_value = 0;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isInstanceOf");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in isInstanceOf");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &typeName);
  /* Copying the Python string to IOR controlled memory */
  typeName = typeName ? strcpy(malloc(strlen(typeName) + 1), typeName) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = !strcmp(((PetscObject) _self_ior)->class_name, typeName);
  /* Convert PetscTruth to PetscTruth */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local__return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) typeName);
  return _pyReturn;
}

static PyObject *PETSc_Vec_setSizes(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "n",
    (char *) "N",
    NULL};
  long _okay = 0;
  int32_t n = 0;
  long _local_n = 0;
  int32_t N = 0;
  long _local_N = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setSizes");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setSizes");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "ll", _kwlist, &_local_n, &_local_N);
  /* Convert long to int32_t */
  n = _local_n;
  /* Convert long to int32_t */
  N = _local_N;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecSetSizes(_self_ior, n, N);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_dot(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "y",
    NULL};
  long _okay = 0;
  Vec y = 0;
  int ierr = 0;
  double _return_value = 0.0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for dot");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in dot");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_Vec__extract_ior, &y);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecDot(_self_ior, y, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "d", _return_value);
  return _pyReturn;
}

static PyObject *PETSc_Vec_tDot(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "y",
    NULL};
  long _okay = 0;
  Vec y = 0;
  int ierr = 0;
  double _return_value = 0.0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for tDot");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in tDot");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_Vec__extract_ior, &y);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecTDot(_self_ior, y, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "d", _return_value);
  return _pyReturn;
}

static PyObject *PETSc_Vec_norm(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "normType",
    NULL};
  long _okay = 0;
  NormType normType = 0;
  long _local_normType = 0;
  int ierr = 0;
  double _return_value = 0.0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for norm");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in norm");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "l", _kwlist, &_local_normType);
  /* Convert long to NormType */
  normType = _local_normType;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecNorm(_self_ior, normType, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "d", _return_value);
  return _pyReturn;
}

static PyObject *PETSc_Vec_normalize(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  double _return_value = 0.0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for normalize");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in normalize");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecNormalize(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "d", _return_value);
  return _pyReturn;
}

static PyObject *PETSc_Vec_sum(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  double _return_value = 0.0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for sum");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in sum");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecSum(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "d", _return_value);
  return _pyReturn;
}

static PyObject *PETSc_Vec_max(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  int32_t position = 0;
  long _local_position = 0;
  double value = 0.0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for max");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in max");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecMax(_self_ior, &position, &value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local_position = position;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "(ld)", _local_position, value);
  return _pyReturn;
}

static PyObject *PETSc_Vec_min(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  int32_t position = 0;
  long _local_position = 0;
  double value = 0.0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for min");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in min");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecMin(_self_ior, &position, &value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local_position = position;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "(ld)", _local_position, value);
  return _pyReturn;
}

static PyObject *PETSc_Vec_scale(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "value",
    NULL};
  long _okay = 0;
  double value = 0.0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for scale");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in scale");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "d", _kwlist, &value);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecScale(_self_ior, value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_copy(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "y",
    NULL};
  long _okay = 0;
  Vec y = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for copy");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in copy");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_Vec__extract_ior, &y);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecCopy(_self_ior, y);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_set(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "value",
    NULL};
  long _okay = 0;
  double value = 0.0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for set");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in set");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "d", _kwlist, &value);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecSet(_self_ior, value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_swap(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "y",
    NULL};
  long _okay = 0;
  Vec y = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for swap");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in swap");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_Vec__extract_ior, &y);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecSwap(_self_ior, y);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_AXPY(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "a",
    (char *) "x",
    NULL};
  long _okay = 0;
  double a = 0.0;
  Vec x = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for AXPY");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in AXPY");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "dO&", _kwlist, &a, (void *) PETSc_Vec__extract_ior, &x);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecAXPY(_self_ior, a, x);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_AYPX(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "a",
    (char *) "x",
    NULL};
  long _okay = 0;
  double a = 0.0;
  Vec x = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for AYPX");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in AYPX");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "dO&", _kwlist, &a, (void *) PETSc_Vec__extract_ior, &x);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecAYPX(_self_ior, a, x);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_AXPBY(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "a",
    (char *) "b",
    (char *) "x",
    NULL};
  long _okay = 0;
  double a = 0.0;
  double b = 0.0;
  Vec x = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for AXPBY");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in AXPBY");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "ddO&", _kwlist, &a, &b, (void *) PETSc_Vec__extract_ior, &x);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecAXPBY(_self_ior, a, b, x);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_MAXPY(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "a",
    (char *) "x",
    NULL};
  long _okay = 0;
  struct petsc__array a;
  struct petsc__array x;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for MAXPY");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in MAXPY");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&", _kwlist, (void *) double__extract_array_borrow, &a, (void *) PETSc_Vec__extract_array_copy, &x);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecMAXPY(_self_ior, a.size, a.data, x.data);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_WAXPY(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "a",
    (char *) "x",
    (char *) "y",
    NULL};
  long _okay = 0;
  double a = 0.0;
  Vec x = 0;
  Vec y = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for WAXPY");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in WAXPY");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "dO&O&", _kwlist, &a, (void *) PETSc_Vec__extract_ior, &x, (void *) PETSc_Vec__extract_ior, &y);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecWAXPY(_self_ior, a, x, y);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_pointwiseMax(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "x",
    (char *) "y",
    NULL};
  long _okay = 0;
  Vec x = 0;
  Vec y = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for pointwiseMax");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in pointwiseMax");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&", _kwlist, (void *) PETSc_Vec__extract_ior, &x, (void *) PETSc_Vec__extract_ior, &y);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecPointwiseMax(_self_ior, x, y);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_pointwiseMaxAbs(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "x",
    (char *) "y",
    NULL};
  long _okay = 0;
  Vec x = 0;
  Vec y = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for pointwiseMaxAbs");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in pointwiseMaxAbs");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&", _kwlist, (void *) PETSc_Vec__extract_ior, &x, (void *) PETSc_Vec__extract_ior, &y);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecPointwiseMaxAbs(_self_ior, x, y);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_pointwiseMin(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "x",
    (char *) "y",
    NULL};
  long _okay = 0;
  Vec x = 0;
  Vec y = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for pointwiseMin");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in pointwiseMin");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&", _kwlist, (void *) PETSc_Vec__extract_ior, &x, (void *) PETSc_Vec__extract_ior, &y);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecPointwiseMin(_self_ior, x, y);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_pointwiseMult(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "x",
    (char *) "y",
    NULL};
  long _okay = 0;
  Vec x = 0;
  Vec y = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for pointwiseMult");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in pointwiseMult");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&", _kwlist, (void *) PETSc_Vec__extract_ior, &x, (void *) PETSc_Vec__extract_ior, &y);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecPointwiseMult(_self_ior, x, y);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_pointwiseDivide(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "x",
    (char *) "y",
    NULL};
  long _okay = 0;
  Vec x = 0;
  Vec y = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for pointwiseDivide");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in pointwiseDivide");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&", _kwlist, (void *) PETSc_Vec__extract_ior, &x, (void *) PETSc_Vec__extract_ior, &y);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecPointwiseDivide(_self_ior, x, y);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_maxPointwiseDivide(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "y",
    NULL};
  long _okay = 0;
  Vec y = 0;
  int ierr = 0;
  double _return_value = 0.0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for maxPointwiseDivide");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in maxPointwiseDivide");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_Vec__extract_ior, &y);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecMaxPointwiseDivide(_self_ior, y, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "d", _return_value);
  return _pyReturn;
}

static PyObject *PETSc_Vec_shift(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "value",
    NULL};
  long _okay = 0;
  double value = 0.0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for shift");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in shift");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "d", _kwlist, &value);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecShift(_self_ior, value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_reciprocal(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for reciprocal");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in reciprocal");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecReciprocal(_self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_sqrt(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for sqrt");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in sqrt");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecSqrt(_self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_abs(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for abs");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in abs");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecAbs(_self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_duplicate(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  Vec _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for duplicate");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in duplicate");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecDuplicate(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) PETSc_Vec__wrap_ior, _return_value);
  if (_return_value)
  {
    PetscObjectReference(((PetscObject) _return_value));
  }
  return _pyReturn;
}

static PyObject *PETSc_Vec_getPetscMap(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  PetscMap _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getPetscMap");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getPetscMap");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecGetPetscMap(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) PETSc_PetscMap__wrap_ior, _return_value);
  if (_return_value)
  {
    PetscObjectReference(((PetscObject) _return_value));
  }
  return _pyReturn;
}

static PyObject *PETSc_Vec_setValues(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "indices",
    (char *) "values",
    (char *) "mode",
    NULL};
  long _okay = 0;
  struct petsc__array indices;
  struct petsc__array values;
  InsertMode mode = 0;
  long _local_mode = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setValues");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setValues");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&l", _kwlist, (void *) int__extract_array_borrow, &indices, (void *) double__extract_array_borrow, &values, &_local_mode);
  /* Convert long to InsertMode */
  mode = _local_mode;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecSetValues(_self_ior, indices.size, indices.data, values.data, mode);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_assemblyBegin(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for assemblyBegin");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in assemblyBegin");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecAssemblyBegin(_self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_assemblyEnd(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for assemblyEnd");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in assemblyEnd");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecAssemblyEnd(_self_ior);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_getBlockSize(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getBlockSize");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getBlockSize");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecGetBlockSize(_self_ior, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *PETSc_Vec_setBlockSize(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "size",
    NULL};
  long _okay = 0;
  int32_t size = 0;
  long _local_size = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setBlockSize");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setBlockSize");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "l", _kwlist, &_local_size);
  /* Convert long to int32_t */
  size = _local_size;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecSetBlockSize(_self_ior, size);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_setValuesBlocked(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "indices",
    (char *) "values",
    (char *) "mode",
    NULL};
  long _okay = 0;
  struct petsc__array indices;
  struct petsc__array values;
  InsertMode mode = 0;
  long _local_mode = 0;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setValuesBlocked");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in setValuesBlocked");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&O&l", _kwlist, (void *) int__extract_array_borrow, &indices, (void *) double__extract_array_borrow, &values, &_local_mode);
  /* Convert long to InsertMode */
  mode = _local_mode;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecSetValuesBlocked(_self_ior, indices.size, indices.data, values.data, mode);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *PETSc_Vec_getArray(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  long _okay = 0;
  int ierr = 0;
  struct petsc__array values;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getArray");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in getArray");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecGetArray(_self_ior, (double **) &(values.data));
  values.size = _self_ior->n;
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) double__wrap_array, &values);
  return _pyReturn;
}

static PyObject *PETSc_Vec_restoreArray(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "values",
    NULL};
  long _okay = 0;
  struct petsc__array values;
  int ierr = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for restoreArray");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in restoreArray");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) double__extract_array_borrow, &values);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecRestoreArray(_self_ior, (double **) &(values.data));
  values.size = _self_ior->n;
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) double__wrap_array, &values);
  return _pyReturn;
}

static PyObject *PETSc_Vec_equal(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  PyObject *_pyReturn = NULL;
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "y",
    NULL};
  long _okay = 0;
  Vec y = 0;
  int ierr = 0;
  PetscTruth _return_value = 0;
  PetscTruth _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for equal");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in equal");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_Vec__extract_ior, &y);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to PETSc */
  ierr = VecEqual(_self_ior, y, &_return_value);
  if (ierr)
  {
    {
      PyObject *errorCode = NULL;
      
      errorCode = PyInt_FromLong(ierr);
      PyErr_SetObject(PyExc_RuntimeError, errorCode);
      Py_DECREF(errorCode);
    }
    return _pyReturn;
  }
  /* Convert PetscTruth to PetscTruth */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local__return_value);
  return _pyReturn;
}

static PyObject *PETSc_Vec_Vec(PyObject *_type, PyObject *_args, PyObject *_kwdict)
{
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  long _okay = 0;
  Vec obj = 0;
  PetscTruth created = 0;
  Vec _return_value = 0;
  PyObject *_pyReturn = NULL;
  
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "|O&", _kwlist, (void *) PETSc_Vec__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  if (obj)
  {
    created = 0;
  }
  else
  {
    created = 1;
    VecCreate(PETSC_COMM_WORLD, &obj);
  }
  _return_value = obj;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) PETSc_Vec__wrap_ior, _return_value);
  if (_return_value)
  {
    PetscObjectReference(((PetscObject) _return_value));
  }
  if (created)
  {
    PetscObjectDereference((PetscObject) obj);
  }
  return _pyReturn;
}

static PyObject *isSame(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  /* Retrieve the IOR pointer */
  Vec _self_ior = ((struct PETSc_Vec_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  long _okay = 0;
  PetscObject obj = 0;
  PetscTruth _return_value = 0;
  PetscTruth _local__return_value = 0;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isSame");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null PETSc object in isSame");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) PETSc_PetscObject__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = (_self_ior == (Vec) obj);
  /* Convert PetscTruth to PetscTruth */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _local__return_value);
  return _pyReturn;
}

static struct PyMethodDef _VecObjectMethods[] = {
  {(char *) "getType", (PyCFunction) PETSc_PetscObject_getType, (METH_VARARGS | METH_KEYWORDS), (char *) "getType()\n\
RETURNS (return string _return_value)\n\
 Get the implementation type"},
  {(char *) "setType", (PyCFunction) PETSc_PetscObject_setType, (METH_VARARGS | METH_KEYWORDS), (char *) "setType(in string typeName)\n\
RETURNS None\n\
\n\
 Set the implementation type"},
  {(char *) "getOptionsPrefix", (PyCFunction) PETSc_PetscObject_getOptionsPrefix, (METH_VARARGS | METH_KEYWORDS), (char *) "getOptionsPrefix()\n\
RETURNS (return string _return_value)\n\
 Get the prefix appended to options directed to this object"},
  {(char *) "setOptionsPrefix", (PyCFunction) PETSc_PetscObject_setOptionsPrefix, (METH_VARARGS | METH_KEYWORDS), (char *) "setOptionsPrefix(in string prefix)\n\
RETURNS None\n\
\n\
 Set the prefix appended to options directed to this object"},
  {(char *) "appendOptionsPrefix", (PyCFunction) PETSc_PetscObject_appendOptionsPrefix, (METH_VARARGS | METH_KEYWORDS), (char *) "appendOptionsPrefix(in string prefix)\n\
RETURNS None\n\
\n\
 Enlarge the prefix appended to options directed to this object"},
  {(char *) "setFromOptions", (PyCFunction) PETSc_PetscObject_setFromOptions, (METH_VARARGS | METH_KEYWORDS), (char *) "setFromOptions()\n\
RETURNS None\n\
\n\
 Configure the object using the PETSc options database"},
  {(char *) "setUp", (PyCFunction) PETSc_PetscObject_setUp, (METH_VARARGS | METH_KEYWORDS), (char *) "setUp()\n\
RETURNS None\n\
\n\
 Initiate the object setup phase"},
  {(char *) "view", (PyCFunction) PETSc_PetscObject_view, (METH_VARARGS | METH_KEYWORDS), (char *) "view(in PETSc.PetscViewer viewer)\n\
RETURNS None\n\
\n\
 Output a representation of the object using "},
  {(char *) "getObject", (PyCFunction) PETSc_PetscObject_getObject, (METH_VARARGS | METH_KEYWORDS), (char *) "getObject()\n\
RETURNS (return opaque _return_value)\n\
 Get the PETSc object pointer"},
  {(char *) "setObject", (PyCFunction) PETSc_PetscObject_setObject, (METH_VARARGS | METH_KEYWORDS), (char *) "setObject(in opaque object)\n\
RETURNS None\n\
\n\
 Set the PETSc object pointer"},
  {(char *) "getReferenceCount", (PyCFunction) ASE_BaseInterface_getReferenceCount, (METH_VARARGS | METH_KEYWORDS), (char *) "getReferenceCount()\n\
RETURNS (return int _return_value)\n\
 Retrieve the intrinsic reference count of the underlying object"},
  {(char *) "addReference", (PyCFunction) ASE_BaseInterface_addReference, (METH_VARARGS | METH_KEYWORDS), (char *) "addReference()\n\
RETURNS None\n\
\n\
 Add one to the intrinsic reference count of the underlying object - Objects continue to exist as long as the reference count is positive - Clients should call this method whenever they create another ongoing reference to an object or interface"},
  {(char *) "deleteReference", (PyCFunction) ASE_BaseInterface_deleteReference, (METH_VARARGS | METH_KEYWORDS), (char *) "deleteReference()\n\
RETURNS None\n\
\n\
 Decrease by one the intrinsic reference count of the underlying object - Delete the object if the reference count is non-positive - Clients should call this method whenever they remove a reference to an object or interface"},
  {(char *) "getClassName", (PyCFunction) ASE_BaseInterface_getClassName, (METH_VARARGS | METH_KEYWORDS), (char *) "getClassName()\n\
RETURNS (return string _return_value)\n\
 Return the type name of the underlying class object"},
  {(char *) "getName", (PyCFunction) ASE_BaseInterface_getName, (METH_VARARGS | METH_KEYWORDS), (char *) "getName()\n\
RETURNS (return string _return_value)\n\
 Retrieve the object name"},
  {(char *) "setName", (PyCFunction) ASE_BaseInterface_setName, (METH_VARARGS | METH_KEYWORDS), (char *) "setName(in string name)\n\
RETURNS None\n\
\n\
 Set the object name"},
  {(char *) "isInstanceOf", (PyCFunction) ASE_BaseInterface_isInstanceOf, (METH_VARARGS | METH_KEYWORDS), (char *) "isInstanceOf(in string typeName)\n\
RETURNS (return bool _return_value)\n\
 Return <code>true</code> if this object is an instance of the specified type - This routine will return <code>true</code> if and only if a cast to the specified type would succeed"},
  {(char *) "setSizes", (PyCFunction) PETSc_Vec_setSizes, (METH_VARARGS | METH_KEYWORDS), (char *) "setSizes(in int n, in int N)\n\
RETURNS None\n\
\n\
 Set the local and global storage sizes"},
  {(char *) "dot", (PyCFunction) PETSc_Vec_dot, (METH_VARARGS | METH_KEYWORDS), (char *) "dot(in PETSc.Vec y)\n\
RETURNS (return double _return_value)\n\
 result = y^H self"},
  {(char *) "tDot", (PyCFunction) PETSc_Vec_tDot, (METH_VARARGS | METH_KEYWORDS), (char *) "tDot(in PETSc.Vec y)\n\
RETURNS (return double _return_value)\n\
 result = y^T self"},
  {(char *) "norm", (PyCFunction) PETSc_Vec_norm, (METH_VARARGS | METH_KEYWORDS), (char *) "norm(in PETSc.NormType normType)\n\
RETURNS (return double _return_value)\n\
 result = ||self||"},
  {(char *) "normalize", (PyCFunction) PETSc_Vec_normalize, (METH_VARARGS | METH_KEYWORDS), (char *) "normalize()\n\
RETURNS (return double _return_value)\n\
 result = ||self|| and vector is normalized"},
  {(char *) "sum", (PyCFunction) PETSc_Vec_sum, (METH_VARARGS | METH_KEYWORDS), (char *) "sum()\n\
RETURNS (return double _return_value)\n\
 result = \\sum_i self_i"},
  {(char *) "max", (PyCFunction) PETSc_Vec_max, (METH_VARARGS | METH_KEYWORDS), (char *) "max(out int position, out double value)\n\
RETURNS (out int position, out double value)\n\
 Returns the maximum \"value\", which is found at index \"position\""},
  {(char *) "min", (PyCFunction) PETSc_Vec_min, (METH_VARARGS | METH_KEYWORDS), (char *) "min(out int position, out double value)\n\
RETURNS (out int position, out double value)\n\
 Returns the minimum \"value\", which is found at index \"position\""},
  {(char *) "scale", (PyCFunction) PETSc_Vec_scale, (METH_VARARGS | METH_KEYWORDS), (char *) "scale(in double value)\n\
RETURNS None\n\
\n\
 self_i = self_i * value"},
  {(char *) "copy", (PyCFunction) PETSc_Vec_copy, (METH_VARARGS | METH_KEYWORDS), (char *) "copy(in PETSc.Vec y)\n\
RETURNS None\n\
\n\
 Copy self into y"},
  {(char *) "set", (PyCFunction) PETSc_Vec_set, (METH_VARARGS | METH_KEYWORDS), (char *) "set(in double value)\n\
RETURNS None\n\
\n\
 self_i = value ERROR: NOT CHANGED IN PETSc YET"},
  {(char *) "swap", (PyCFunction) PETSc_Vec_swap, (METH_VARARGS | METH_KEYWORDS), (char *) "swap(in PETSc.Vec y)\n\
RETURNS None\n\
\n\
 Swap the values in self and y"},
  {(char *) "AXPY", (PyCFunction) PETSc_Vec_AXPY, (METH_VARARGS | METH_KEYWORDS), (char *) "AXPY(in double a, in PETSc.Vec x)\n\
RETURNS None\n\
\n\
 self_i = a * x_i + self_i ERROR: NOT CHANGED IN PETSc YET"},
  {(char *) "AYPX", (PyCFunction) PETSc_Vec_AYPX, (METH_VARARGS | METH_KEYWORDS), (char *) "AYPX(in double a, in PETSc.Vec x)\n\
RETURNS None\n\
\n\
 self_i = x_i + a * self_i"},
  {(char *) "AXPBY", (PyCFunction) PETSc_Vec_AXPBY, (METH_VARARGS | METH_KEYWORDS), (char *) "AXPBY(in double a, in double b, in PETSc.Vec x)\n\
RETURNS None\n\
\n\
 self_i = a * x_i + b * self_i"},
  {(char *) "MAXPY", (PyCFunction) PETSc_Vec_MAXPY, (METH_VARARGS | METH_KEYWORDS), (char *) "MAXPY(in array a, in array x)\n\
RETURNS None\n\
\n\
 self_i = \\sum_i a_i x_i + self_i"},
  {(char *) "WAXPY", (PyCFunction) PETSc_Vec_WAXPY, (METH_VARARGS | METH_KEYWORDS), (char *) "WAXPY(in double a, in PETSc.Vec x, in PETSc.Vec y)\n\
RETURNS None\n\
\n\
 self_i = a x_i + y_i"},
  {(char *) "pointwiseMax", (PyCFunction) PETSc_Vec_pointwiseMax, (METH_VARARGS | METH_KEYWORDS), (char *) "pointwiseMax(in PETSc.Vec x, in PETSc.Vec y)\n\
RETURNS None\n\
\n\
 self_i = max(x_i, y_i)"},
  {(char *) "pointwiseMaxAbs", (PyCFunction) PETSc_Vec_pointwiseMaxAbs, (METH_VARARGS | METH_KEYWORDS), (char *) "pointwiseMaxAbs(in PETSc.Vec x, in PETSc.Vec y)\n\
RETURNS None\n\
\n\
 self_i = max(|x_i|, |y_i|)"},
  {(char *) "pointwiseMin", (PyCFunction) PETSc_Vec_pointwiseMin, (METH_VARARGS | METH_KEYWORDS), (char *) "pointwiseMin(in PETSc.Vec x, in PETSc.Vec y)\n\
RETURNS None\n\
\n\
 self_i = min(x_i, y_i)"},
  {(char *) "pointwiseMult", (PyCFunction) PETSc_Vec_pointwiseMult, (METH_VARARGS | METH_KEYWORDS), (char *) "pointwiseMult(in PETSc.Vec x, in PETSc.Vec y)\n\
RETURNS None\n\
\n\
 self_i = x_i y_i"},
  {(char *) "pointwiseDivide", (PyCFunction) PETSc_Vec_pointwiseDivide, (METH_VARARGS | METH_KEYWORDS), (char *) "pointwiseDivide(in PETSc.Vec x, in PETSc.Vec y)\n\
RETURNS None\n\
\n\
 self_i = x_i / y_i"},
  {(char *) "maxPointwiseDivide", (PyCFunction) PETSc_Vec_maxPointwiseDivide, (METH_VARARGS | METH_KEYWORDS), (char *) "maxPointwiseDivide(in PETSc.Vec y)\n\
RETURNS (return double _return_value)\n\
 result = max_i |x_i / y_i|"},
  {(char *) "shift", (PyCFunction) PETSc_Vec_shift, (METH_VARARGS | METH_KEYWORDS), (char *) "shift(in double value)\n\
RETURNS None\n\
\n\
 self_i = self_i + value"},
  {(char *) "reciprocal", (PyCFunction) PETSc_Vec_reciprocal, (METH_VARARGS | METH_KEYWORDS), (char *) "reciprocal()\n\
RETURNS None\n\
\n\
 self_i = 1 / self_i"},
  {(char *) "sqrt", (PyCFunction) PETSc_Vec_sqrt, (METH_VARARGS | METH_KEYWORDS), (char *) "sqrt()\n\
RETURNS None\n\
\n\
 self_i = \\sqrt{self_i}"},
  {(char *) "abs", (PyCFunction) PETSc_Vec_abs, (METH_VARARGS | METH_KEYWORDS), (char *) "abs()\n\
RETURNS None\n\
\n\
 self_i = |self_i|"},
  {(char *) "duplicate", (PyCFunction) PETSc_Vec_duplicate, (METH_VARARGS | METH_KEYWORDS), (char *) "duplicate()\n\
RETURNS (return PETSc.Vec _return_value)\n\
 Duplicates the vector (does not copy values)"},
  {(char *) "getPetscMap", (PyCFunction) PETSc_Vec_getPetscMap, (METH_VARARGS | METH_KEYWORDS), (char *) "getPetscMap()\n\
RETURNS (return PETSc.PetscMap _return_value)\n\
 Return the layout of the vector"},
  {(char *) "setValues", (PyCFunction) PETSc_Vec_setValues, (METH_VARARGS | METH_KEYWORDS), (char *) "setValues(in array indices, in array values, in PETSc.InsertMode mode)\n\
RETURNS None\n\
\n\
 Set values into the vector"},
  {(char *) "assemblyBegin", (PyCFunction) PETSc_Vec_assemblyBegin, (METH_VARARGS | METH_KEYWORDS), (char *) "assemblyBegin()\n\
RETURNS None\n\
\n\
 Start parallel assembly"},
  {(char *) "assemblyEnd", (PyCFunction) PETSc_Vec_assemblyEnd, (METH_VARARGS | METH_KEYWORDS), (char *) "assemblyEnd()\n\
RETURNS None\n\
\n\
 Finish parallel assembly"},
  {(char *) "getBlockSize", (PyCFunction) PETSc_Vec_getBlockSize, (METH_VARARGS | METH_KEYWORDS), (char *) "getBlockSize()\n\
RETURNS (return int _return_value)\n\
 Get the block size"},
  {(char *) "setBlockSize", (PyCFunction) PETSc_Vec_setBlockSize, (METH_VARARGS | METH_KEYWORDS), (char *) "setBlockSize(in int size)\n\
RETURNS None\n\
\n\
 Set the block size"},
  {(char *) "setValuesBlocked", (PyCFunction) PETSc_Vec_setValuesBlocked, (METH_VARARGS | METH_KEYWORDS), (char *) "setValuesBlocked(in array indices, in array values, in PETSc.InsertMode mode)\n\
RETURNS None\n\
\n\
 Set values into the vector"},
  {(char *) "getArray", (PyCFunction) PETSc_Vec_getArray, (METH_VARARGS | METH_KEYWORDS), (char *) "getArray()\n\
RETURNS (return array values)\n\
 Get the local storage"},
  {(char *) "restoreArray", (PyCFunction) PETSc_Vec_restoreArray, (METH_VARARGS | METH_KEYWORDS), (char *) "restoreArray(inout array values)\n\
RETURNS (inout array values)\n\
 Return the local storage"},
  {(char *) "equal", (PyCFunction) PETSc_Vec_equal, (METH_VARARGS | METH_KEYWORDS), (char *) "equal(in PETSc.Vec y)\n\
RETURNS (return bool _return_value)\n\
 result = (self == y)"},
  {(char *) "isSame", (PyCFunction) isSame, (METH_VARARGS | METH_KEYWORDS), (char *) "isSame(in PETSc.PetscObject obj)\n\
RETURNS (return bool _return_value)\n\
"},
  {NULL, NULL}};

static struct PyMethodDef _VecModuleMethods[] = {{NULL, NULL}};

static void sidl_dealloc(struct PETSc_Vec_PythonObject *self)
{
  if (self->weakRefList)
  {
    PyObject_ClearWeakRefs((PyObject *) self);
  }
  if ((self->refType != WEAK_REF) && (self->ior))
  {
    VecDestroy(self->ior);
  }
  self->ob_type->tp_free(self);
}

static int sidl_cmp(struct PETSc_Vec_PythonObject *self, PyObject *arg)
{
  Vec ior1 = self->ior;
  PetscObject ior2 = (PetscObject) ((struct PETSc_Vec_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (arg->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for sidl_cmp");
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  _return_value = PyObject_Compare(o1, o2);
  Py_DECREF(o1);
  Py_DECREF(o2);
  return _return_value;
}

static long sidl_hash(struct PETSc_Vec_PythonObject *self)
{
  Vec ior = self->ior;
  PyObject *_hash_arg = NULL;
  long _return_value = 0;
  
  if (!ior)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_hash()");
    return _return_value;
  }
  _hash_arg = Py_BuildValue((char *) "i", ior);
  if (!_hash_arg)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not hash IOR pointer in sidl_hash()");
    return _return_value;
  }
  _return_value = PyObject_Hash(_hash_arg);
  Py_DECREF(_hash_arg);
  return _return_value;
}

static PyObject *sidl_str(struct PETSc_Vec_PythonObject *self)
{
  const char *_str = NULL;
  PyObject *_return_value = NULL;
  Vec ior = self->ior;
  
  VecGetType(ior, (VecType *) &_str);
  _return_value = PyString_FromString(_str);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyObject *sidl_richcmp(struct PETSc_Vec_PythonObject *self, PyObject *arg, int op)
{
  Vec ior1 = self->ior;
  PetscObject ior2 = (PetscObject) ((struct PETSc_Vec_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  PyObject *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (!PyObject_HasAttrString(arg, (char *) "getClassName"))
  {
    switch(op) {
      case Py_EQ:
        _return_value = Py_False;
        Py_INCREF(_return_value);
        break;
      case Py_NE:
        _return_value = Py_True;
        Py_INCREF(_return_value);
        break;
      case Py_LT:
      case Py_LE:
      case Py_GT:
      case Py_GE:
        PyErr_SetString(PyExc_TypeError, (char *) "sidl_richcmp: Cannot order non-PETSc object");
    }
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  _return_value = PyObject_RichCompare(o1, o2, op);
  Py_DECREF(o1);
  Py_DECREF(o2);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyTypeObject SIDLPythonObjectType = {
  PyObject_HEAD_INIT(NULL)
  0,
  (char *) "PETSc.Vec",
  sizeof(struct PETSc_Vec_PythonObject),
  0,
  (destructor) sidl_dealloc,
  (printfunc) 0,
  (getattrfunc) 0,
  (setattrfunc) 0,
  (cmpfunc) sidl_cmp,
  (reprfunc) 0,
  (PyNumberMethods *) 0,
  (PySequenceMethods *) 0,
  (PyMappingMethods *) 0,
  (hashfunc) sidl_hash,
  (ternaryfunc) 0,
  (reprfunc) sidl_str,
  (getattrofunc) 0,
  (setattrofunc) 0,
  (PyBufferProcs *) 0,
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_WEAKREFS,
  (char *) "This is a Python wrapper for a SIDL object of type PETSc.Vec",
  (traverseproc) 0,
  (inquiry) 0,
  (richcmpfunc) sidl_richcmp,
  (int) &((struct PETSc_Vec_PythonObject *) 0)->weakRefList,
  (getiterfunc) 0,
  (iternextfunc) 0,
  _VecObjectMethods,
  (struct PyMemberDef *) 0,
  (struct PyGetSetDef *) 0,
  0,
  0,
  (descrgetfunc) 0,
  (descrsetfunc) 0,
  0,
  (initproc) 0,
  (allocfunc) 0,
  (newfunc) PETSc_Vec_Vec,
  (freefunc) 0,
  (inquiry) 0};

void initVec(void)
{
  PyObject *module = NULL;
  
  /* Setup the python type */
  if (PyType_Ready(&SIDLPythonObjectType) < 0)
  {
    return;
  }
  /* Initialize the module */
  module = Py_InitModule3((char *) "Vec", _VecModuleMethods, (char *) "");
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Could not initialize module PETSc.Vec");
    return;
  }
  /* Add the type to the module and increment the ref count */
  Py_INCREF(&SIDLPythonObjectType);
  PyModule_AddObject(module, (char *) "Vec", (PyObject *) &SIDLPythonObjectType);
  /* Import Numeric */
  import_array();
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Cannot import Numeric Python array module");
    return;
  }
}
